/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading;

/** OMG Trading Service imports. */
import org.omg.CosTrading.Admin;
import org.omg.CosTrading.Proxy;
import org.omg.CosTrading.OfferIdIterator;
import org.omg.CosTrading.ProxyPackage.ProxyInfo;
import org.omg.CosTrading.OfferIdSeqHolder;
import org.omg.CosTrading.OfferIdIteratorHolder;

/**
 * Wrapper class used to provide a node in the administration console
 * tree which children are
 * <code>org.omg.CosTrading.ProxyPackage.ProxyInfo</code> instances.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class ProxyOffers {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** 
     * The CosTrading Admin interface used to obtain the CosTrading
     * Proxies OfferId. 
     */
    protected Admin admin_;

    /** 
     * The CosTrading Proxy interface used to obtain the CosTrading
     * ProxyInfo references from OfferIds. 
     */
    protected Proxy proxy_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor. 
     *
     * @param admin The CosTrading Admin reference of the trader which
     * contains Offers to view.  
     */
    public ProxyOffers(Admin admin) 
    {
        admin_ = admin;
        proxy_ = admin.proxy_if();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Returns all the CosTrading Proxy Offers contained in the
     * trader. This method retrieves offers 100 by 100.
     *
     * @return All CosTrading Proxy Offers contained in the trader.
     */
    public ProxyInfo[] getOffers()
    {
        return getOffers(100);
    }

    /**
     * Returns all the CosTrading proxy Offers contained in the
     * trader.
     *
     * @param how_many_step The step used to retrieve offers.
     * @return All CosTrading proxy Offers contained in the trader.
     */
    public ProxyInfo[] getOffers(int how_many_step)
    {
        // Retrieving offer id list 
        java.util.List oid_list = new java.util.LinkedList();
        OfferIdSeqHolder oid_seq = new OfferIdSeqHolder();
        OfferIdIteratorHolder oid_iter = new OfferIdIteratorHolder();
        try {            

            // performing the distant "list_offers" op.
            admin_.list_proxies(how_many_step, oid_seq, oid_iter);

            // Storing offer ids returned in the sequence.
            if (oid_seq.value.length == 0) return new ProxyInfo[0];
            for (int i=0 ; i<oid_seq.value.length ; i++) {
                oid_list.add(oid_seq.value[i]);
            }

            // Storing offer ids in the iterator.
            if (oid_iter.value != null) {
                OfferIdIterator iter = oid_iter.value;
                boolean again = iter.next_n(how_many_step, oid_seq);
                {
                    for (int i=0 ; i<oid_seq.value.length ; i++) {
                        oid_list.add(oid_seq.value[i]);
                    }                    
                    again = iter.next_n(how_many_step, oid_seq);
                } while (again);                
                iter.destroy();
            }                        
        } catch (org.omg.CosTrading.NotImplemented ex) {
            // Returning empty array
            return new ProxyInfo[0];
        }

        // Constructing the ProxyInfo seq
        java.util.List info_list = new java.util.LinkedList();
        for (java.util.Iterator iter = oid_list.iterator() ; iter.hasNext() ; )
        {
            String oid = (String)iter.next();
            try {
                ProxyInfo info = proxy_.describe_proxy(oid);
                info_list.add(info);
            } catch (org.omg.CosTrading.IllegalOfferId ex) {
                // should not happened 
            } catch (org.omg.CosTrading.UnknownOfferId ex) {
                // the offer has been destroyed during the session 
                // try the next one
            } catch (org.omg.CosTrading.ProxyPackage.NotProxyOfferId ex) {
                // should not happened until Trader bug
            }
        }

        return (ProxyInfo[])info_list.toArray(new ProxyInfo[0]);
    }

}
