/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading;

/** OMG CosTrading imports. */
import org.objectweb.openccm.corba.trader.SpecifiedPropsWrapper;
import org.omg.CosTrading.Policy;
import org.omg.CosTrading.Lookup;
import org.omg.CosTrading.Offer;

import javax.swing.JOptionPane;

/**
 * Wrapper class used to provide a node in the administration console
 * tree which children are <code>org.omg.CosTrading.Offer</code>
 * instances which results of the query execution.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class Query {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The label of the query. */
    protected String label_;

    /** 
     * The CosTrading Lookup interface used to execute the query.
     */
    protected Lookup lookup_;

    /** The service type to address. */
    protected String service_type_;

    /** The constraint to use. */
    protected String constraint_;

    /** The preference to use in the query. */
    protected String pref_;

    /** The policies to use in the query. */
    protected Policy[] policies_;

    /** The parent queries wrapper object. */
    protected Queries parent_;

    private boolean removed = false;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor. 
     *
     * @param query_label The label of the query.
     * @param lookup The Lookup reference used to do the query.
     * @param service_type The service type name to address in the query.
     * @param constraint The constraint to use in the query.
     * @param pref The preference to use in the query.
     * @param policies The policies to apply in the query.
     */
    public Query(String query_label, Queries parent, String service_type, 
                 String constraint, String pref, Policy[] policies)
    {
        label_ = query_label;
        parent_ = parent;
        lookup_ = parent.getLookup();
        service_type_ = service_type;
        constraint_ = constraint;
        pref_ = pref;
        policies_ = policies;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Used to treat an error while evaluating the query. Common
     * action is to display an error message and to remove this query
     * from his parent.
     *
     * @param message The error message to display.
     */
    protected void fatalError(String message) {
        removed = true;
        JOptionPane.showMessageDialog(null, message, "Error on query '" + label_ + "'", JOptionPane.ERROR_MESSAGE);
        parent_.removeQuery(this);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Returns the CosTrading Offers selected by the query.
     *
     * @return The CosTrading Offers selected by the query.
     */
    public Offer[] getOffers() {
        return getOffers(100);
    }

    /**
     * Returns the CosTrading Offers selected by the query.
     *
     * @param how_many_step The step used to retrieve offers. 
     * @return The CosTrading Offers selected by the query.
     */
    public Offer[] getOffers(int how_many_step) {

        if (removed) return new Offer[0];

        // Construct a SpecifedProp 
        org.omg.CosTrading.LookupPackage.SpecifiedProps cos_props;
        cos_props = SpecifiedPropsWrapper.create_all();

        // Construct the used holders
        org.omg.CosTrading.OfferSeqHolder offerSeq;
        offerSeq = new org.omg.CosTrading.OfferSeqHolder();
        org.omg.CosTrading.OfferIteratorHolder offerIter;
        offerIter = new org.omg.CosTrading.OfferIteratorHolder();
        org.omg.CosTrading.PolicyNameSeqHolder limits;
        limits = new org.omg.CosTrading.PolicyNameSeqHolder();

        // Perform the query
        try {
            lookup_.query(service_type_, constraint_, pref_, policies_, 
                         cos_props, how_many_step, offerSeq, offerIter, limits);

            // Compute the results of the query
            java.util.Vector offerAnswer = new java.util.Vector();
            for (int i = 0 ; i < offerSeq.value.length ; i++) {
                org.omg.CosTrading.Offer offer = (offerSeq.value)[i];
                offerAnswer.add(offer);
            }
      
            // Checking the iterator
            if (offerIter.value != null) {
                org.omg.CosTrading.OfferSeqHolder offerSeqIter = new org.omg.CosTrading.OfferSeqHolder();
                boolean more;
                do {
                    more = offerIter.value.next_n(how_many_step, offerSeqIter);
                    for (int i = 0 ; i < offerSeqIter.value.length ; i++) {
                        org.omg.CosTrading.Offer offer = (offerSeqIter.value)[i];
                        offerAnswer.add(offer);
                    }
                } while(more);
                offerIter.value.destroy();
            }
            return (Offer[]) offerAnswer.toArray(new Offer[0]);

        } catch (org.omg.CosTrading.IllegalServiceType ex) {
            fatalError("bad service type name '" + service_type_ + "'.");
        } catch (org.omg.CosTrading.UnknownServiceType ex) {
            fatalError("bad service type name '" + service_type_ + "'.");
        } catch (org.omg.CosTrading.IllegalConstraint ex) {
            fatalError("bad constraint '" + constraint_ + "'.");
        } catch (org.omg.CosTrading.LookupPackage.IllegalPreference ex) {
            fatalError("bad preference '" + pref_ + "'.");
        } catch (org.omg.CosTrading.LookupPackage.IllegalPolicyName ex) {
            fatalError("bad policy name '" + ex.name + "'.");
        } catch (org.omg.CosTrading.LookupPackage.PolicyTypeMismatch ex) {
            fatalError("bad policy type for '" + ex.the_policy.name + "'.");
        } catch (org.omg.CosTrading.LookupPackage.InvalidPolicyValue ex) {
            fatalError("bad policy value for '" + ex.the_policy.name + "'.");
        } catch (org.omg.CosTrading.IllegalPropertyName ex) {
            fatalError("bad property name '" + ex.name + "'.");
        } catch (org.omg.CosTrading.DuplicatePropertyName ex) {
            fatalError("duplicated property name '" + ex.name + "'.");
        } catch (org.omg.CosTrading.DuplicatePolicyName ex) {
            fatalError("duplicated policy name " + ex.name + ".");
        }      
        return new Offer[0];
    }

    /**
     * Returns the Lookup reference to use to do the query.
     *
     * @return The Lookup reference to use to do the query.
     */
    public Lookup getLookup() {
        return lookup_;
    }

    /**
     * Returns the label of the query.
     *
     * @return The label of the query.
     */
    public String getQueryLabel() {
        return label_;
    }

    /**
     * Returns the service type name used in the query.
     *
     * @return The service type name used in the query.
     */
    public String getServiceTypeName() {
        return service_type_;
    }

    /**
     * Returns the constraint used in the query.
      *
     * @return The constraint used in the query.
    */
    public String getConstraint() {
        return constraint_;
    }

    /**
     * Returns the preference used in the query.
     *
     * @return The preference used in the query.
     */
    public String getPref() {
        return pref_;
    }

    /**
     * Returns the policies used in the query.
     *
     * @return The policies used in the query.
     */
    public Policy[] getPolicies() {
        return policies_;
    }

}
