/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.objectweb.openccm.explorer.util.ior;

import java.util.ArrayList;
import java.util.Arrays;
import org.omg.CORBA.StringHolder;

public final class IorPrinter  {
    private static final char CHARS[] = { ';', '/', ':', '?', '@', '&', '=', '+', '$', ',', '-', '_', '.', '!', '~', '*', '`', '(', ')' };
    private static final char NOT_ESCAPED[] = Sort(CHARS);
    private static char[] Sort(char chars[]) {
        Arrays.sort(chars);
        return chars;
    }

    private boolean little_endian_;
    //byte order
    private String type_id_;
    //repository type id
    private ArrayList iprofiles_ = new ArrayList();
    //InternetProfiles
    private ArrayList profiles_ = new ArrayList();
    //all profiles (TaggedProfiles and InternetProfiles)
    private static boolean IsPrintableCharacter(char c) {
        return ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z') || ('0' <= c && c <= '9') || Arrays.binarySearch(NOT_ESCAPED, c) >= 0;
    }

    private byte[] GetBytes(String str, int offset) {
        int nbytes = str.length() - offset;

        if ((nbytes & 1) == 1) {
            throw new org.omg.CORBA.INV_OBJREF("Bad string length");
        }

        nbytes /= 2;
        byte bytes[] = new byte[nbytes];

        for (int i = 0, j = 4; i < nbytes; i++, j += 2) {
            String bstr = str.substring(j, j + 2);

            try {
                bytes[i] = (byte)Integer.parseInt(bstr, 16);
            } catch (NumberFormatException exc) {
                throw new org.omg.CORBA.INV_OBJREF("Bad byte value: " + bstr);
            }
        }

        return bytes;
    }

    ///////////////////////////////////////////////////////////////////////
    // Methods read IORs, respectively, in "IOR:" and "corbaloc:" notations.
    ///////////////////////////////////////////////////////////////////////
    private void Read_ior(String ior) {
        Reset();
        byte bytes[] = GetBytes(ior, 4);

        try {
            ModestInputStream in = new ModestInputStream(bytes);
            little_endian_ = in.GetEndian();
            type_id_ = in.ReadString();
            int nprofiles = in.ReadULong();

            for (int i = 0; i < nprofiles; i++) {
                int tag = in.ReadULong();
                byte profile_data[] = in.ReadOctetArray();

                if (tag == org.omg.IOP.TAG_INTERNET_IOP.value) {
                    InternetProfile iprofile = new InternetProfile();
                    iprofile.Read_ior(profile_data);
                    iprofiles_.add(iprofile);
                    profiles_.add(iprofile);
                } else {
                    org.omg.IOP.TaggedProfile profile = new org.omg.IOP.TaggedProfile();
                    profile.tag = tag;
                    profile.profile_data = profile_data;
                    profiles_.add(profile);
                }
            }
        } catch (ArrayIndexOutOfBoundsException exc) {
            throw new org.omg.CORBA.INV_OBJREF("Invalid IOR (too short?)");
        }
    }

    private void Read_corbaloc(String ior) {
        Reset();
        int index = ior.indexOf(':') + 1;
        //here reference starts
        boolean lastaddr = false;

        do {
            int ind = index + 2, addrend = ior.indexOf(',', ind);

            if (addrend == - 1) {
                addrend = ior.indexOf('/', ind);
                lastaddr = true;
            }

            if (addrend == - 1) {
                throw new org.omg.CORBA.INV_OBJREF("Bad reference: no '/'<key_string>");
            }
            InternetProfile profile = new InternetProfile();

            profile.Read_corbaloc(ior.substring(index, addrend));
            iprofiles_.add(profile);
            profiles_.add(profile);
            index = addrend + 1;
        } while (!lastaddr);
        byte objkey[] = index < ior.length() ? Decode(ior.substring(index)) : new byte[0];

        for (int i = 0; i < iprofiles_.size(); i++) {
            ((InternetProfile)iprofiles_.get(i)).object_key = objkey;
        //IOR returns copies
        }

        type_id_ = "IDL:omg.org/CORBA/Object:1.0";
    }

    ///////////////////////////////////////////////////////////////////////
    // Methods deal with "corbaloc:" notation.
    ///////////////////////////////////////////////////////////////////////
    private boolean CheckIfCorbalocPossible(StringHolder strh) {
        // check if any additional profiles
        if (iprofiles_.size() != profiles_.size()) {
            if (strh != null) {
                strh.value = "Additional profiles present in reference";
            }

            return false;
        }
        // check if InternetProfiles don't contain components
        // and if object keys are the same in all profiles.
        InternetProfile previous = null;

        for (int i = 0; i < iprofiles_.size(); i++) {
            InternetProfile iprofile = (InternetProfile)iprofiles_.get(i);

            if (iprofile.components != null && iprofile.components.length > 0) {
                if (strh != null) {
                    strh.value = "Components present in Internet Profiles";
                }

                return false;
            }

            if (i > 0) {
                if (!Arrays.equals(iprofile.object_key, previous.object_key)) {
                    if (strh != null) {
                        strh.value = "Different object keys in Internet Profiles";
                    }

                    return false;
                }
            }

            previous = iprofile;
        }

        return true;
    }

    private String Write_corbaloc() {
        StringBuffer sb = new StringBuffer(512);
        byte objkey[] = null;
        sb.append("corbaloc:");

        for (int i = 0; i < iprofiles_.size(); i++) {
            InternetProfile profile = (InternetProfile)iprofiles_.get(i);

            if (i == 0) {
                objkey = profile.object_key;
            } else {
                sb.append(',');
            }

            profile.Write_corbaloc(sb);
        }

        sb.append('/');
        sb.append(Encode(objkey));
        return sb.toString();
    }

    ///////////////////////////////////////////////////////////////////////
    // Constructors.
    ///////////////////////////////////////////////////////////////////////
    public IorPrinter() {
    }

    public IorPrinter(String ior) {

        Read(ior);
    }

    ///////////////////////////////////////////////////////////////////////
    // Reset and access methods.
    ///////////////////////////////////////////////////////////////////////
    public synchronized void Reset() {
        little_endian_ = false;
        type_id_ = null;
        iprofiles_.clear();
        profiles_.clear();
    }

    public synchronized boolean IsLittleEndian() {
        return little_endian_;
    }

    public synchronized void SetLittleEndian(boolean le) {
        little_endian_ = le;
    }

    public synchronized String getTypeId() {
        return type_id_;
    }

    public synchronized void SetTypeId(String type_id) {
        type_id_ = type_id;
    }

    public synchronized int GetInternetProfilesCount() {
        return iprofiles_.size();
    }

    public synchronized InternetProfile GetInternetProfile(int index) {
        return new InternetProfile((InternetProfile)iprofiles_.get(index));
    }

    public synchronized void SetInternetProfile(int index, InternetProfile iprofile) {
        InternetProfile iprof = (InternetProfile)iprofiles_.get(index);
        iprof.Set(iprofile);
    }

    public synchronized void AddInternetProfile(InternetProfile iprofile) {
        iprofile = new InternetProfile(iprofile);
        iprofiles_.add(iprofile);
        profiles_.add(iprofile);
    }

    public synchronized void InsertInternetProfile(int index, InternetProfile iprofile) {
        iprofile = new InternetProfile(iprofile);
        iprofiles_.add(index, iprofile);

        if (index > 0) {
            Object object = iprofiles_.get(index - 1);
            index = profiles_.indexOf(object) + 1;
        }

        profiles_.add(index, iprofile);
    }

    public synchronized void RemoveInternetProfile(int index) {
        Object object = iprofiles_.remove(index);
        profiles_.remove(object);
    }

    ///////////////////////////////////////////////////////////////////////
    // Method reads reference.
    ///////////////////////////////////////////////////////////////////////
    public synchronized void Read(String ior) {
        ior = ior.trim();
        int len = ior.length();

        if (len < 4) {
            //"IOR:".length()
            throw new org.omg.CORBA.INV_OBJREF("Object reference too short");
        } else if (ior.substring(0, 4).compareToIgnoreCase("IOR:") == 0) {
            Read_ior(ior);
        } else if (len < 9) {
            //"corbaloc:".length()
            throw new org.omg.CORBA.INV_OBJREF("Object reference too short or invalid");
        } else if (ior.substring(0, 9).compareToIgnoreCase("corbaloc:") == 0) {
            Read_corbaloc(ior);
        } else {
            throw new org.omg.CORBA.INV_OBJREF("Object reference should start with \"IOR:\" or \"corbaloc:\"");
        }
    }

    ///////////////////////////////////////////////////////////////////////
    // Methods write IORs, respectively, in "IOR:", "corbaloc:" and default
    // notation.
    ///////////////////////////////////////////////////////////////////////
    public synchronized String toString_ior() {
        ModestOutputStream out = new ModestOutputStream(little_endian_);
        out.WriteString(type_id_);
        out.WriteULong(profiles_.size());

        for (int i = 0; i < profiles_.size(); i++) {
            Object object = profiles_.get(i);

            if (object instanceof InternetProfile) {
                out.WriteULong(org.omg.IOP.TAG_INTERNET_IOP.value);
                out.WriteOctetArray(((InternetProfile)object).Write_ior(little_endian_));
            } else {
                org.omg.IOP.TaggedProfile profile = (org.omg.IOP.TaggedProfile)object;
                out.WriteULong(profile.tag);
                out.WriteOctetArray(profile.profile_data);
            }
        }
        byte bytes[] = out.GetBytes();
        String str = "IOR:";

        for (int i = 0; i < bytes.length; i++) {
            String hexstr = Integer.toHexString((int)bytes[i] & 0xFF);

            if (hexstr.length() == 1) {
                hexstr = '0' + hexstr;
            }

            str += hexstr;
        }

        return str;
    }

    public synchronized String toString_corbaloc() throws UnsupportedOperationException {
        StringHolder strh = new StringHolder();

        if (!CheckIfCorbalocPossible(strh)) {
            throw new UnsupportedOperationException(strh.value);
        }

        return Write_corbaloc();
    }

    // Return reference in default format: the preferred one is user-readable
    // "corbaloc:" notation; if not possible (addition profiles, components in
    // InternetProfiles or different object keys in profiles), returns "IOR:".
    public synchronized String toString() {
        return CheckIfCorbalocPossible(null) ? toString_corbaloc() : toString_ior();
    }

    ///////////////////////////////////////////////////////////////////////
    // Various describing methods.
    ///////////////////////////////////////////////////////////////////////
    public synchronized String Describe() {
        StringBuffer sb = new StringBuffer(512);
        sb.append("byte order: " + (little_endian_ ? "little" : "big") + " endian; type_id: " + type_id_ + ';');

        for (int i = 0; i < iprofiles_.size(); i++) {
            InternetProfile profile = (InternetProfile)iprofiles_.get(i);
            sb.append(" [IIOPv. " + profile.major + '.' + profile.minor + "; " + "host: " + profile.host + ':' + profile.port + "; object key: ");
            byte object_key[] = profile.object_key;

            for (int j = 0; j < object_key.length; j++) {
                String hexstr = Integer.toHexString((int)object_key[j] & 0xFF);
                sb.append(' ');

                if (hexstr.length() == 1) {
                    sb.append('0');
                }

                sb.append(hexstr);
            }

            sb.append(']');
        }

        return sb.toString();
    }

    ///////////////////////////////////////////////////////////////////////
    // Methods perform mapping from '%HH' notation to plain byte array and vice-versa
    ///////////////////////////////////////////////////////////////////////
    public static byte[] Decode(String str) {
        byte bytes[] = new byte[str.length()];
        int blen = 0;

        for (int i = 0, len = str.length(); i < len; ) {
            char c = str.charAt(i++);

            if (c == '%') {
                bytes[blen++] = (byte)Integer.parseInt(str.substring(i, i + 2), 16);
                i += 2;
            } else {
                bytes[blen++] = (byte)c;
            }
        }

        if (blen < bytes.length) {
            byte b[] = new byte[blen];
            System.arraycopy(bytes, 0, b, 0, b.length);
            bytes = b;
        }

        return bytes;
    }

    public static String Encode(byte bytes[]) {
        StringBuffer sb = new StringBuffer(3 * bytes.length);

        for (int i = 0; i < bytes.length; ) {
            char c = (char)bytes[i++];

            if (IsPrintableCharacter(c)) {
                sb.append(c);
            } else {
                String hex = Integer.toHexString((int)c & 0xFF);
                sb.append('%');

                if (hex.length() == 1) {
                    sb.append('0');
                }

                sb.append(hex);
            }
        }

        return sb.toString();
    }

    public static void printIOR(String iorstr) {
        try {
            IorPrinter ior = new IorPrinter(iorstr);
            System.out.println(ior.Describe());
        } catch (Throwable exc) {
            exc.printStackTrace();
        }
    }

    public String getPort() {
        InternetProfile profile = (InternetProfile)iprofiles_.get(0);
        int port = profile.port;

        if (port < 0) {
            port += 65536;
        }

        return "" + port;
    }

    public String getHost() {
        InternetProfile profile = (InternetProfile)iprofiles_.get(0);
        return profile.host;
    }

    public String getVersion() {
        InternetProfile profile = (InternetProfile)iprofiles_.get(0);
        return "" + (int)profile.major + "." + (int)profile.minor;
    }

    public void print() {
        InternetProfile profile = (InternetProfile)iprofiles_.get(0);
        System.out.println("------IOR components-----");
        System.out.println("TypeId:       " + getTypeId());
        System.out.println("IIOP Version: " + "" + (int)profile.major + "." + (int)profile.minor);
        System.out.println("Host:         " + profile.host);
        System.out.println("Port:         " + profile.port);
    }

    public String getProfileId(long tag) {
        if (tag == 0) {
            return "TAG_INTERNET_IOP";
        } else {
            return "TAG_MULTIPLE_COMPONENTS";
        }
    }

    private String toHex(byte bs[]) {
        String hex = "";

        for (int i = 0; i < bs.length; i++) {
            int n1 = (bs[i] & 0xff) / 16;
            int n2 = (bs[i] & 0xff) % 16;
            char c1 = (char)(n1 > 9 ? ('A' + (n1 - 10)) : ('0' + n1));
            char c2 = (char)(n2 > 9 ? ('A' + (n2 - 10)) : ('0' + n2));
            hex += ((i > 0) ? " " : "") + c1 + c2;
        }

        return hex;
    }
}
