package org.coach.idltree;

import org.w3c.dom.Node;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import org.coach.util.IorPrinter;

/**
 * The class IdlObject represents an CORBA IDL Object value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * The value is maintained as a stringified ior value.
 * An IdlObject object has no child nodes.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlObject extends IdlNode implements IdlWritable {
    protected IdlObject() {
        setUserObject(this);
        type = "Object";
        value = "null";
        tc = orb.get_primitive_tc(TCKind.tk_objref);
    }

    protected IdlObject(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlObject(TypeCode tc) {
        this();
        setNode(tc);
    }
    
    protected void setNode(Any any) {
        try {         
            setNode(any.type());           
//            org.omg.CORBA.Object obj = any.create_input_stream().read_Object();
            org.omg.CORBA.Object obj = any.extract_Object();
            try {
                value = orb.object_to_string(obj);
                IorPrinter iorPrinter = new IorPrinter(value);
                id = iorPrinter.getTypeId();
            } catch (Exception e) {
                value = "null";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            any.type(tc);
            if (!value.equals("null")) {
                any.insert_Object(orb.string_to_object(value));
            }
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Assigns the node value from a string.
     *
     * @param v The string from which to convert the value.
     */
    public void setValue(String v) {
        if (v.toUpperCase().equals("NULL")) {
            value = "null";
            id = "";
        } else {
            int idx = v.indexOf("IOR:");
            if (idx < 0) {
                throw new RuntimeException("value is not an IOR: " + v);
            }
            value = v.substring(idx);
            try {
                IorPrinter iorPrinter = new IorPrinter(value);
                id = iorPrinter.getTypeId();
            } catch (Exception e) {
                id = "";
            }
        }
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            os.write_Object(orb.string_to_object(value));
        } catch (Exception e) {
            e.printStackTrace();
        }            
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        value = orb.object_to_string(is.read_Object());
    }
    
    // XML section

    /**
     * Create an IdlObject node from an Xml representation.
     *
     * XML format example:
     * <pre>
     *   &lt;object&gt;id="IDL:examples/MyInterface:1.0"&gt;
     *       IOR:000....
     *   &lt;/object&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlObject instance.
     */
    public IdlObject(String xml) {
        this(XmlNode.getNode(xml));
    }
    
    /**
     * Construct an XmlNode from an Xml node.
     * Package access only. Called from the Xml parser to contruct
     * an Xml type tree from an Xml representation.
     */
    IdlObject(Node n) {
        this();
        if (n == null || !n.getNodeName().toUpperCase().equals("OBJECT")) {
            throw new RuntimeException("Object expected");
        }
        setValue(XmlNode.getText(n));
    } 
  
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        w.write_Object(value);
    }
}