package org.coach.idltree;

import org.w3c.dom.Node;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import java.util.*;
import org.coach.tracing.api.*;
import java.lang.reflect.*;

/**
 * The class IdlReply represents a reply of an operation on an CORBA IDL interface. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlReply object has child nodes for each out, inout or return parameter.
 * An IdlReply object can also contain an exception value as the result of an operation.
 * The method isException() can be used to determine if the IdlReply object contains an exception value.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlReply extends IdlNode implements IdlWritable {
    private String[] allTypes;
    private String[] allNames;
    private String[] allDirections;
    protected boolean isException;
    protected String name;
    protected String[] names = new String[0];
    protected String[] exceptions = new String[0];
    protected Parameter[] params = new Parameter[0];
    protected transient TypeCode[] types = new TypeCode[0];
    protected transient TypeCode[] exceptionTypes = new TypeCode[0];
    protected boolean hasReturn;

    protected IdlReply() {
        isLeaf = false;
        isMutable = false;
        setUserObject(this);
        type = "reply";
    }
                        
    /**
     * Constructs an IdlReply node.
     *
     * The IdlReply node is initialized with IdlNode parameter children from the values array.
     *
     * @param id The interface repository id for the interface to which this operation belongs.
     * @param name The operation name.
     * @param p The array with parameter name, parameter type and direction information.
     * @param values The any values used to initialize the operation parameters.
     */
    public IdlReply(String id, String name, Parameter[] p, Any[] values) {
        this();
        try {
            this.id = id;
            this.name = name;
            setField(name);
            if (p != null) {
                params = p;
                names = new String[p.length];
                for (int i = 0; i < p.length; i++) {
                    if (p[i].dir.equals("return")) {
                        p[i].name = "return";
                        if(!p[i].type.equals("void")) {
                            hasReturn = true;
                        }
                    }
                    names[i] = p[i].name;
                    add(new IdlParameter(p[i].dir, p[i].name, values[i]));
                }
            }
            types = new org.omg.CORBA.TypeCode[values.length];
            for (int i = 0; i < values.length; i++) {
                types[i] = values[i].type();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Constructs an IdlReply node with exceptions.
     *
     * The IdlReply node is initialized with IdlNode parameter children from the values array.
     *
     * @param id The interface repository id for the interface to which this operation belongs.
     * @param name The operation name.
     * @param p The array with parameter name, parameter type and direction information.
     * @param values The any values used to initialize the operation parameters.
     * @param exceptions The list of exception names for this reply.
     * @param exceptionTypes The list of exception typecodes.     
     */
    public IdlReply(String id, String name, Parameter[] p, Any[] values, String[] exceptions, TypeCode[] exceptionTypes) {
        this(id, name, p, values);
        this.exceptions = exceptions;
        this.exceptionTypes = exceptionTypes;
        types = new org.omg.CORBA.TypeCode[values.length];
        for (int i = 0; i < values.length; i++) {
            types[i] = values[i].type();
        }
    }

    /**
     * Constructs an IdlReply node with an exception value.
     *
     * The IdlReply node is initialized with IdlNode parameter children from the values array.
     *
     * @param id The interface repository id for the interface to which this operation belongs.
     * @param name The operation name.
     * @param exValue The any containing the exception for this reply. 
     */
    public IdlReply(String id, String name, Any exValue) {
        this();
        try {
            this.id = id;
            this.name = name;
            setField(name);
            this.isException = true;
            IdlNode n = new IdlException(exValue);
            n.field = "exception";
            hasReturn = false;
            add(n);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
        
    /**
     * Return the name of the operation this IdlReply object represents.
     *
     * @return The operation name.
     */ 
    public String getName() {
        return name;
    }
    
    /**
     * Return the list of parameter names for this reply.
     *
     * @return The list of parameter names.
     */  
    public String[] getParameterNames() {
        return names;
    }


    /**
     * Return the list of parameter descriptions for this reply.
     *
     * @return The list of parameter descriptions.
     */
    public Parameter[] getParameterInfo() {
        return params;
    }

    /**
     * Return the list of exceptionr names for this reply.
     *
     * @return The list of exception names.
     */
    public String[] getExceptions() {
        return exceptions;
    }
            
    /**
     * Return the IdlNode child for a named parameter.
     *
     * @param name The parameter name for which to return the child IdlNode.
     *
     * @return The IdlNode child for the parameter indicated name.
     */ 
    public IdlNode getParameter(String name) {
        for (int i = 0; i < names.length; i++) {
            if (names[i].equals(name)) {
                return ((IdlParameter)getChildAt(i)).getParameter();
            }
        }
        return null;
    }

    /**
     * Sets the current exception from an index number in the exception list.
     *
     * @param i The index number in the exception list.
     *
     */ 
    public void setException(int i) {
        try {
            if (i < 0) {
                value = "no exception";
                setDefaults();
            } else {
                isException = true;
                removeAllChildren();
                IdlNode n = new IdlException(exceptionTypes[i]);
                n.field = "exception";
                value = exceptions[i];
                hasReturn = false;
                add(n);
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }              
    } 

    /**
     * Sets the current exception from the content of an any.
     *
     * @param exValue The any contain the exception type.
     */ 
    public void setException(org.omg.CORBA.Any exValue) {
        try {
            IdlNode n = new IdlException(exValue);
            n.field = "exception";
            value = exValue.type().id();
            removeAllChildren();
            isException = true;
            hasReturn = false;
            add(n);
        } catch (Throwable t) {
            t.printStackTrace();
        }              
    }
    
    protected void setDefaults() {
        removeAllChildren();
        isException = false;
        for (int i = 0; i < params.length; i++) {
            add(new IdlParameter(params[i].dir, params[i].name, create(types[i]).toAny()));
        }
    }
    
    /**
     * Tests if the IdlReply contains an exception value.
     *
     * @return True is the IdlReply contains an exception, false otherwise.
     */
    public boolean isException() {
        return isException;
    }

    /**
     * Tests if the IdlReply has a return value.
     *
     * @return True is the IdlReply has a return value, false otherwise.
     */
    public boolean hasReturn() {
        return hasReturn;
    }
            
    /**
     * Return the exception value of the IdlReply as an IdlNode.
     *
     * @return The exception value of the IdlReply as an IdlNode.
     */
    public IdlNode getException() {
        if (isException && getChildCount() == 1) {
            return (IdlNode)getChildAt(0);
        }
        return null;
    }
    
    /**
     * Return all IdlNode child nodes.
     * 
     * @return An IdlNode array containing all in and inout parameter values.
     */ 
    public IdlNode[] getParameters() {
        IdlNode[] members = new IdlNode[getChildCount()];
        for(int i = 0; i < members.length; i++) {
            members[i] = ((IdlParameter)getChildAt(i)).getParameter();
        }
        return members;
    }

    /**
     * Returns a string representation for this IdlReply node.
     *
     * @return The string representation for this IdlReply node.
     */
    public String toString() {
        return "reply " + id + " " + name;
    } 
        
    /**
     * Fills the given any array with parameter values.
     *
     * The content of the values array is filled with the parameter
     * child nodes converted to any values. The given array length must
     * match the number of in and inout parameter values for this reply.
     */
    public void getValues(Any[] values) {
        if (values.length != getChildCount()) {
            throw new RuntimeException("Invalid array length for any values");
        }
        for(int i = 0; i < getChildCount(); i++) {
            values[i] = ((IdlParameter)getChildAt(i)).getParameter().toAny();
        } 
    }

    /**
     * Sets the IdlReply values from the content of the given any array with parameter values.
     *
     * The content of the values array is used to initialize the child nodes with the parameter
     * child nodes converted from any values. The given array length must
     * match the number of in and inout parameter values for this reply.
     */
    public void setValues(Any[] values) {
        if (values.length != params.length) {
            throw new RuntimeException("Value array length (" + values.length + ") does not match number of parameters (" + params.length + ")");
        }
        removeAllChildren();
        for (int i = 0; i < params.length; i++) {
            add(new IdlParameter(params[i].dir, params[i].name, values[i]));
        }
    }
    
    /**
     * Reads all inout, out and return value from an InputStream.
     * 
     * The result is returned as an Any array.
     *
     * @param in The org.omg.CORBA.portable.InputStream to read the values from.
     *
     * @return An any array with inout, out and return values read from the inputstream.
     */
    public Any[] readValues(org.omg.CORBA.portable.InputStream in) {
        IdlNode[] replyParameters = getParameters();
        Any[] outVal = new Any[replyParameters.length];

        try {
            if (hasReturn) {
                // The return value is the first value in the stream but the last value
                // in the parameter array.
                org.omg.CORBA.TypeCode ptc = replyParameters[replyParameters.length - 1].getTypeCode();
                outVal[replyParameters.length - 1] = orb.create_any();
                outVal[replyParameters.length - 1].read_value(in, ptc);
                for (int i = 0; i < replyParameters.length - 1; i++) {             
                    outVal[i] = orb.create_any();
                    ptc = replyParameters[i].getTypeCode();
                    outVal[i].read_value(in, ptc);
                }
            } else {                               
                for (int i = 0; i < replyParameters.length; i++) {             
                    outVal[i] = orb.create_any();
                    org.omg.CORBA.TypeCode ptc = replyParameters[i].getTypeCode();
                    outVal[i].read_value(in, ptc);
                }
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }
        setValues(outVal);
        
        return outVal;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        for(int i = 0; i < getChildCount(); i++) {
            ((IdlParameter)getChildAt(i)).write(os);
        }
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        for(int i = 0; i < getChildCount(); i++) {
            ((IdlParameter)getChildAt(i)).read(is);
        } 
    }
        
    // XML section

    /**
     * Create an IdlReply from an Xml representation.
     *
     * The XML format example
     * <pre>
     * &lt;reply id="IDL:example/MyInterface:1.0"&gt;
     *  &lt;inout name="p2"&gt;
     *   &lt;long&gt;100&lt;/long&gt;
     *  &lt;/inout&gt;
     *  &lt;return name="p2"&gt;
     *   &lt;string&gt;my return value&lt;/string&gt;
     *  &lt;/return&gt;
     * &lt;/reply&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlReply instance.
     */
    public IdlReply(String xml) {
        this(XmlNode.getNode(xml));
    }

    IdlReply(Node n) {
        this();
        try {
            id = XmlNode.getId(n);
            name = XmlNode.getName(n);
            setField(name);
            
            initParameters(id);
            
            removeAllChildren();

            Node[] nodes = XmlNode.childElements(n);
            if (nodes.length == 1 && nodes[0].getNodeName().toUpperCase().equals("EXCEPTION")) {
                add(XmlNode.getIdlNode(nodes[0]));
                hasReturn = false;
            } else {
                for (int i = 0; i < nodes.length; i++) {
                    if (!names[i].equals("return") && !names[i].equals(XmlNode.getName(nodes[i]))) {
                        throw new RuntimeException("Unknown parameter name: " + XmlNode.getName(nodes[i]) +  " for " + id + " " + name + " expected name: " + names[i]);
                    }
                    add(new IdlParameter(nodes[i]));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e.toString());
        }
    }

    /**
     * Constructs an intialized IdlReply node from an interface repository id and operation name.
     *
     * @param id The interface repository id.
     * @param name The operation name.
     */
    public IdlReply(String id, String name) {
        this();
        try {
            // We use reflection here and rely on the presence of the HelperExt class.
            // Alternatively the IR server can be used.
            this.id = id;
            this.name = name;
            setField(name);

            initParameters(id);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void initParameters(String id) {
        try {
            allDirections = XmlNode.getParameterDirections(id, name);
            allTypes = XmlNode.getParameterTypes(id, name);
            allNames = XmlNode.getParameterNames(id, name);
            exceptions = XmlNode.getParameterExceptions(id, name);

            exceptionTypes = new org.omg.CORBA.TypeCode[exceptions.length];
            if (exceptions.length > 0) {
                isMutable = true;
                value = "no exception";
                for (int i = 0; i < exceptions.length; i++) {
                    exceptionTypes[i] = XmlNode.type(exceptions[i]);
                }
            }            

            removeAllChildren();
            for (int i = 0; i < allNames.length; i++) {
                if (allDirections[i].endsWith("out") || (allDirections[i].equals("return") && !allTypes[i].equals("void"))) {
                    add(new IdlParameter(allDirections[i], allNames[i], XmlNode.type(allTypes[i])));
                }
            }

            // The names and params array only have vales for  non void return, out and inout parameters
            int idx = getChildCount();
            names = new String[idx];
            params = new Parameter[idx];
            types = new org.omg.CORBA.TypeCode[idx];
            idx = 0; 
            for (int i = 0; i < allNames.length; i++) {
                if (allDirections[i].endsWith("out") || (allDirections[i].equals("return") && !allTypes[i].equals("void"))) {
                    params[idx] = new Parameter(allDirections[i], allTypes[i], allNames[i]);
                    names[idx] = allNames[i];
                    types[idx] = XmlNode.type(allTypes[i]);
                    idx++;
                }
                if (allDirections[i].equals("return") && !allTypes[i].equals("void")) {
                    hasReturn = true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlReply n, IdlWriter w) {
        w.write_start_reply(n.getName(), n.getId());
        for(int i = 0; i < n.getChildCount(); i++) {
            XmlNode.write((IdlNode)n.getChildAt(i), w);
        }        
        w.write_end_reply();
    }
}