/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class IdlTreePanel extends JPanel {
    private TreePath selection;
    private IdlModel model;
    private IdlNode nodeCopy;
    private BorderLayout borderLayout1 = new BorderLayout();
    private JScrollPane TreeScrollPane = new JScrollPane();
    private JTree typeTree = new JTree();
    private JPopupMenu popupMenu = new JPopupMenu();
    private JMenuItem copyMenuItem = new JMenuItem();
    private JMenuItem pasteMenuItem = new JMenuItem();
    private JMenuItem saveMenuItem = new JMenuItem();
    private JMenuItem cloneMenuItem = new JMenuItem();
    private JFileChooser chooser;
    private JMenuItem openMenuItem = new JMenuItem();
    private TypeSelectorDialog typeSelectorDialog;
    private Frame parentFrame;
    private ObjectListener objectListener;

    /** the list of listeners for the ModelChangeEvent */
    protected Vector modelListeners = new Vector();

    public void addModelListener(ModelChangeListener listener) {
        if (!modelListeners.contains(listener)) {
            modelListeners.addElement(listener);
        }
    }

    public void removeModelListener(ModelChangeListener listener) {
        modelListeners.removeElement(listener);
    }
    
    public void addObjectListener(ObjectListener objectListener) {
        this.objectListener = objectListener;
    }

    public void removeObjectListener(ObjectListener objectListener) {
        this.objectListener = null;
    }

    public void notifyModelListeners() {
        ModelChangeEvent evt = new ModelChangeEvent(model, this);
        ModelChangeListener listener;
        for (int i = 0; i < modelListeners.size(); i++) {
            listener = (ModelChangeListener)modelListeners.elementAt(i);
            listener.modelChanged(evt);
        }
    }
    
    public IdlTreePanel() {
        try  {
            jbInit();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    public IdlTreePanel(IdlModel m) {
        try  {
            jbInit();
            setModel(m);
            IdList.getTree();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public boolean isReply() {
        if (model != null && model.getRoot() instanceof IdlReply) {
            return true;
        }
        return false;
    }
                
    private Frame getFrame() {
        if (parentFrame != null) {
            return parentFrame;
        }
        Component c = this;
        while (c != null && !(c instanceof Frame)) {
            c = c.getParent();
        }
        parentFrame = (Frame)c;
        return parentFrame;
    }
    
    private TypeSelectorDialog getTypeSelectorDialog() {
        if (typeSelectorDialog != null) {
            return typeSelectorDialog;
        }
        typeSelectorDialog = new TypeSelectorDialog();
        return typeSelectorDialog;
    }

    public void setModel(IdlModel tm) {
        model = tm;
        typeTree.setModel(model);
        typeTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        typeTree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                TreePath selection = e.getNewLeadSelectionPath();
                model.setSelection(selection);
            }
        });

        typeTree.setEditable(true);
        TreeCellEditor editor = new TypeNodeEditor(model);
        typeTree.setCellEditor(editor);
        typeTree.validate();
    }
    
    private void updateModel(IdlModel tm) {
        setModel(tm);
        notifyModelListeners();
    }
    
    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        chooser = new JFileChooser(".");
        chooser.setDialogTitle("Save as XML file");
        chooser.setApproveButtonText("Save");
        chooser.setApproveButtonToolTipText("Saving File");
        chooser.setAutoscrolls(true);
        chooser.setDialogType(chooser.SAVE_DIALOG);
        chooser.addChoosableFileFilter(new XML_FileFilter());
        copyMenuItem.setText("copy");
        copyMenuItem.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                copyMenuItem_actionPerformed(e);
            }
        });
        pasteMenuItem.setText("paste");
        pasteMenuItem.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                pasteMenuItem_actionPerformed(e);
            }
        });
        saveMenuItem.setText("save");
        saveMenuItem.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                saveMenuItem_actionPerformed(e);
            }
        });
        typeTree.addMouseListener(new java.awt.event.MouseAdapter() {

            public void mouseClicked(MouseEvent e) {
                showPopup(e);
            }

            public void mousePressed(MouseEvent e) {
                showPopup(e);
            }

            public void mouseReleased(MouseEvent e) {
                showPopup(e);
            }
        });
        cloneMenuItem.setText("clone");
        cloneMenuItem.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cloneMenuItem_actionPerformed(e);
            }
        });

        openMenuItem.setText("open");
        openMenuItem.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                openMenuItem_actionPerformed(e);
            }
        });
        this.add(TreeScrollPane, BorderLayout.CENTER);
        TreeScrollPane.getViewport().add(typeTree, null);
        popupMenu.add(copyMenuItem);
        popupMenu.add(pasteMenuItem);
        popupMenu.add(saveMenuItem);
        popupMenu.add(openMenuItem);
        popupMenu.add(cloneMenuItem);
    }

    void copyMenuItem_actionPerformed(ActionEvent e) {
        IdlNode node = (IdlNode)model.getSelection().getLastPathComponent();
        nodeCopy = node.copy();
    }

    void pasteMenuItem_actionPerformed(ActionEvent e) {
        if (nodeCopy != null && model.getSelection() != null) {
            IdlNode node = (IdlNode)model.getSelection().getLastPathComponent();
            IdlNode parent = (IdlNode)node.getParent();
            if (node.canPaste(nodeCopy)) {
                node.paste(nodeCopy);
                try {
                    model.nodeStructureChanged(node);
                } catch(Exception ex) {
                    JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                }
            } else {
                JOptionPane.showMessageDialog(null, "type mismatch");
            }
        }
    }

    void saveMenuItem_actionPerformed(ActionEvent e) {
        IdlNode node = (IdlNode)model.getRoot();
        int state = chooser.showSaveDialog(null);
        File file = chooser.getSelectedFile();

        if (file != null && state == JFileChooser.APPROVE_OPTION) {
            try {
                XmlWriter w = new XmlWriter();
                XmlNode.write(node, w);
                PrintWriter p = new PrintWriter(new FileWriter(file));
                p.println(w.toString());
                p.close();
//                System.out.println(w.toString());
            } catch (Exception ex) {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(null, ex.toString());
            }
        }
    }

    void showPopup(MouseEvent e) {
        if (e.isPopupTrigger()) {
            popupMenu.show(typeTree, e.getX(), e.getY());
        }
    }

    void cloneMenuItem_actionPerformed(ActionEvent e) {
        IdlNode node = (IdlNode)model.getSelection().getLastPathComponent();
        IdlNode clone = (IdlNode)node.clone();
        if (clone != null) {
//            TypeTreeFrame frame = new TypeTreeFrame(new IdlModel(clone));
//            GUIApp.launch(frame,"AnyTree", 300, 300, 450, 400);
        }
    }

    void openMenuItem_actionPerformed(ActionEvent e) {
        IdlNode node = (IdlNode)model.getRoot();
        int state = chooser.showOpenDialog(null);
        File file = chooser.getSelectedFile();

        if (file != null && state == JFileChooser.APPROVE_OPTION) {
            IdlNode t = null;
            try {
//                t = node.fromAnyFile(file.getPath());
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
            }
            if (t != null) {
                try {
//                    updateModel(new TypeModel(t));
                } catch(Exception ex) {
                    JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                }
            }
        }
    }

    class XML_FileFilter extends javax.swing.filechooser.FileFilter{
        public boolean accept(File f){
            String path = f.getPath().toLowerCase();
            if (path.endsWith(".xml")){
                return true;
            }
            return false;
        }

       public String getDescription(){
            return "XML files (*.xml)";
        }
    }

    /**
     * This class handles the presentation of a node.
     */
    class TypeNodeRenderer extends DefaultTreeCellRenderer {
        private Component strut = Box.createHorizontalStrut(5);
        private Hashtable nodes = new Hashtable();
    
        public TypeNodeRenderer() {
            super();
        }
    
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            JPanel panel;
            IdlNode node = (IdlNode)value;
            Object key = node.getUserObject();
    
            Component cmp = null;
            // to make a node uneditable set cmp to null. The default is a text field
            if (node instanceof IdlUnion) {
                cmp = createComboBox(((IdlUnion)node).getTags(), ((IdlUnion)node).getIndex());
            } else if (node instanceof IdlEnum) {
                cmp = createComboBox(((IdlEnum)node).getEnums(), ((IdlEnum)node).getIndex());
            } else if (node instanceof IdlOperation) {
                cmp = null;
            } else if (node instanceof IdlParameter) {
                cmp = null;
            } else if (node instanceof IdlStruct) {
                cmp = null;
            } else if (node instanceof IdlException) {
                cmp = null;
            } else if (node instanceof IdlReply) {
                String[] exceptions = ((IdlReply)node).getExceptions();
                if (exceptions.length > 0) {
                    // create a combo box with all the exception names plus a "no exception" choice.
                    String[] choices = new String[exceptions.length + 1];
                    choices[0] = "no exception";
                    for (int i = 0; i < exceptions.length; i++) {
                        choices[i + 1] = exceptions[i];
                    }
                    cmp = createComboBox(choices, 0);
                } else {
                    cmp = null;
                }
            } else if (node.getType().equals("boolean")) {
                cmp = createComboBox(new String[] {"FALSE", "TRUE"}, (node.getValue().equals("TRUE") ? 1 : 0));
            } else if (node instanceof IdlAny) {
                cmp = null;

            } else if (node instanceof IdlSequence) {
                if (((IdlSequence)node).isBounded()) {
                    cmp = null;
                } else {
                    JTextField t = new JTextField();
                    t.setColumns(5);
                    t.setText("" + ((IdlSequence)node).getSize());
                    cmp = t;
                }
            } else if (node instanceof IdlObject) {
                JTextField t = new JTextField();
                final String v = node.getValue();
                t.setColumns(10);
                t.setText(v);
                t.setCaretPosition(0);
                cmp = t;
                if (isReply()) {
                    objectAction(v);
                }
            } else {
                JTextField t = new JTextField();
                String v = node.getValue();
                t.setColumns(10);
                t.setText(v);
                t.setCaretPosition(0);
                cmp = t;
            }
            if (cmp != null) {
                nodes.put(key, cmp);
            }
            Component x = super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
    
            JLabel label = new JLabel(node.toString());
            panel = new JPanel();
            setOpaque(false);
            panel.setOpaque(false);
            panel.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
            panel.add(x);
            if (cmp != null) {
                panel.add(strut);
                cmp.setVisible(false);
                panel.add(cmp);
            }
            return panel;
        }
        
        private void objectAction(String ior) {
            if (objectListener != null) {
                objectListener.objectAction(ior);
            }
        }
    
        public Component getEdit(Object value) {
            Object key = ((IdlNode)value).getUserObject();
            return (Component)nodes.get(key);
        }
    
        private JComboBox createComboBox(String[] list, int idx) {
            JComboBox c = new JComboBox();
            for (int i = 0; i < list.length; i++) {
                c.addItem(list[i]);
            }
            c.setSelectedIndex(idx);
            return c;
        }
    }

    /**
     * This class handles editing of a node.
     */
    class TypeNodeEditor extends AbstractCellEditor {
        private DefaultTreeModel model;
        private TypeNodeRenderer renderer;
        private IdlNode node;
        private IdlNode parent;
    
        public TypeNodeEditor(DefaultTreeModel m) {
            model = m;
            renderer = new TypeNodeRenderer();
        }
    
        public Component getTreeCellEditorComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row) {
            node = (IdlNode)value;
    
            Component c = renderer.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, true);
    
            final Component cmp = renderer.getEdit(value);
            final JComboBox combo;
            final JTextField text;
            final JTree idtree;

            if (node instanceof IdlUnion) {
                final IdlUnion union = (IdlUnion)node;
                combo = (JComboBox)cmp;
                combo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        JComboBox src = (JComboBox)e.getSource();
                        int idx = src.getSelectedIndex();
                        try {
                            union.setIndex(idx);
                            model.nodeStructureChanged(union.getParent());
                        } catch (Exception ex) {
                            JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                        }
                        stopCellEditing();
                    }
                });
            }  else if (node instanceof IdlReply) {
                final IdlReply reply = (IdlReply)node;
                String[] exceptions = reply.getExceptions();
                if (exceptions.length > 0) {
                    combo = (JComboBox)cmp;
                    combo.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            JComboBox src = (JComboBox)e.getSource();
                            int idx = src.getSelectedIndex();
                            try {
                                reply.setException(idx - 1);
                                model.nodeStructureChanged(reply);
                            } catch (Exception ex) {
                                JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                            }
                            stopCellEditing();
                        }
                    });
                }
            }  else if (node.getType().equals("boolean")) {
                combo = (JComboBox)cmp;
                combo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        JComboBox src = (JComboBox)e.getSource();
                        int idx = src.getSelectedIndex();
                        try {
                            node.setValue(idx == 0 ? "FALSE" : "TRUE");
                        } catch (Exception ex) {
                            JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                        }
                        stopCellEditing();
                    }
                });
            }  else if (node instanceof IdlEnum) {
                final IdlEnum idlEnum = (IdlEnum)node;
                combo = (JComboBox)cmp;
                combo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        JComboBox src = (JComboBox)e.getSource();
                        int idx = src.getSelectedIndex();
                        try {
                            idlEnum.setIndex(idx);
                        } catch (Exception ex) {
                            JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                        }
                        stopCellEditing();
                    }
                });
            } else if (node instanceof IdlSequence) {
                final IdlSequence seq = (IdlSequence)node;
                if (!seq.isBounded()) {
                    text = (JTextField)cmp;
                    text.addFocusListener(new java.awt.event.FocusAdapter() {
                        public void focusLost(FocusEvent e) {
                            JTextField src = (JTextField)e.getSource();   
                            String size = src.getText();
                            try {
                                seq.setSize(Integer.parseInt(size));
                                model.nodeStructureChanged(seq.getParent());
                            } catch (Exception ex) {
                                JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                            }                      
                            stopCellEditing();
                        }
                    });
                } else {
                    stopCellEditing();
                }
            } else if (node instanceof IdlAny) {
                final IdlAny any = (IdlAny)node;
                TypeSelectorDialog d = getTypeSelectorDialog();
                d.show();
                String id = d.getSelection();
                if (!id.equals("")) {
                    try {
                        any.setContent(id);
                        model.nodeStructureChanged(any.getParent());
                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                    }
                }
                stopCellEditing();
            } else {
                text = (JTextField)cmp;
                if (text != null) {
                    text.addFocusListener(new java.awt.event.FocusAdapter() {
                        public void focusLost(FocusEvent e) {
                            JTextField src = (JTextField)e.getSource();
                            String v = src.getText();
                            try {
                                node.setValue(v);
                            } catch (Exception ex) {
                                JOptionPane.showMessageDialog(null, ex.toString());
ex.printStackTrace();
                            }
                            stopCellEditing();
                        }
                    });
                }
            }
            if (cmp != null) {
                cmp.setVisible(true);
            }
            return c;
        }
    
        public Object getCellEditorValue() {
            return node.getUserObject();
        }
    }
 
    class TypeSelectorDialog extends JDialog {
        private TypeSelector typeSelector;
        
        public TypeSelectorDialog() {
            super(getFrame(), "Select id", true);
            setLocationRelativeTo(getFrame());
            typeSelector = new TypeSelector(this);
            setContentPane(typeSelector);
            pack();
        }

        public String getSelection() {
            return typeSelector.getSelection();
        }
    } 

    class TypeSelector extends JPanel {
        private String selection = "";
        private JButton cancelButton = new JButton("Cancel");
        private JButton okButton = new JButton("Ok");
        private JLabel selectionLabel = new JLabel(" ");
        private JDialog dialog;

        public TypeSelector(JDialog d) {
            dialog = d;
            final JTree idTree = IdList.getTree();
            setLayout(new BorderLayout());
            JScrollPane typeTreeScrollPane = new JScrollPane();       
            add(typeTreeScrollPane, BorderLayout.CENTER);
            typeTreeScrollPane.getViewport().add(idTree, null);
            JPanel buttonPanel = new JPanel();
            buttonPanel.add(okButton);
            buttonPanel.add(cancelButton);
            add(buttonPanel, BorderLayout.SOUTH);
            add(selectionLabel, BorderLayout.NORTH);
            
            cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    selection = "";
                    selectionLabel.setText(" ");
                    dialog.dispose();
                }
            });
            
            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dialog.dispose();
                }
            });
                                   
            idTree.addTreeSelectionListener(new TreeSelectionListener() {
                public void valueChanged(TreeSelectionEvent e) {
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode)idTree.getLastSelectedPathComponent();
                    if (node == null || !node.isLeaf()) {
                        selection = "";
                        selectionLabel.setText(" ");
                        return;
                    }
                    TreeNode[] p = node.getPath();
                    String id = "";
                    String sep = "";
                    for (int i = 1; i < p.length; i++) {
                        id += sep + (String)((DefaultMutableTreeNode)p[i]).getUserObject();
                        sep = "/";
                    }
                    selection = id;
                    selectionLabel.setText(selection);
                }
            });
        }
        
        public String getSelection() {
            return selection;
        }
    }
}
