/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.tcl;

import tcl.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import org.coach.util.IorPrinter;
import org.coach.idltree.*;

public class TclShell extends JPanel  {
    // The command input
    private StringBuffer commandBuffer = new StringBuffer();
    private PrintWriter pout;
    private BufferedReader in;
    private PrintWriter out = new TextAreaPrintWriter(new TextAreaPrintStream());
    private TclLib api;

    // The Tcl interpreter
    private Interp tclInterp = new Interp();;

    // The TextArea widget for displaying commands and results
    public JTextArea textArea;

    // Prompts
    public String mainPrompt = "% ";
    public String contPrompt = "> ";

    // Cursors
    private int promptCursor = 0;
    private int commandCursor = 0;

    // History
    public int historyLength = 10;
    private int historyCursor = 0;
    private Vector historyCommands = new Vector();

    // Platform dependent newline
    private static String newline = System.getProperty("line.separator");
    private String inputText;
    private boolean inputMode;
    private Object lock = new Object();

    public TclShell() {
        this(null, null);
    }
    
    // Create a new Shell instance
    public TclShell(org.omg.CORBA.Object target, String source) {
    	
        // Graphics part
        super(new BorderLayout());
        
        JScrollPane scrollPane = new JScrollPane();
        textArea = new JTextArea("", 20, 80);
        scrollPane.getViewport().add(textArea, null);
        add(scrollPane, BorderLayout.CENTER);

        // Event handling part
        textArea.addKeyListener(new ShellKeyListener());

        try {
            PipedWriter pw1 = new PipedWriter();
            pout = new PrintWriter(pw1, true);
            in = new BufferedReader(new PipedReader(pw1));
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Initialize the interpreter
        try {
            tclInterp.setVar("out", tcl.lang.ReflectObject.newInstance(tclInterp, out.getClass(), out), 0);
            tclInterp.eval("package require java");
            if (target != null) {
                org.omg.CORBA.ORB orb = org.objectweb.openccm.corba.TheORB.getORB();
                String ior = orb.object_to_string(target);
                IorPrinter iorPrinter = new IorPrinter(ior);
                String id = iorPrinter.getTypeId();
                IdlInterface interfaceNode = new IdlInterface(id, false);
                interfaceNode.setValue(ior);    
                tclInterp.setVar("target", tcl.lang.ReflectObject.newInstance(tclInterp, IdlInterface.class, interfaceNode), 0);
            }
            tclInterp.setVar("TclShell", tcl.lang.ReflectObject.newInstance(tclInterp, TclShell.class, this), 0);
            tclInterp.eval("proc puts {s} {global TclShell; $TclShell putText $s\\n}");
//            tclInterp.eval("proc gets {} {global TclShell; return [$TclShell getText]}");

            // Initialze API's functions (tcl procs) tclInterp
            api = new TclLib(tclInterp);

            if (source != null) {
                putText("source \"" + source + "\"");
                evalCommand("source \"" + source + "\"");
                textArea.append("\n");
                promptCursor -= 2;
            }
            
        } catch (tcl.lang.TclException e) {
            System.out.println(tclInterp.getResult());
        }
    }

    public void setOrb(org.omg.CORBA.ORB orb) {
        api.setOrb(orb);
    }

    public BufferedReader getReader() {
        return in;
    }

    public PrintWriter getWriter() {
        return out;
    }

    // Override this to initialize the prompt. We need to do
    // because we can't write to the TextArea until the peer
    // has been created
    public void addNotify() {
        super.addNotify();
        putText(mainPrompt);
    }

    // Print output to the console, update the prompt cursor
    public void putText(String s) {
        if (s.length() != 0) {
            textArea.append(s);
            promptCursor += s.length();
            textArea.setCaretPosition(promptCursor);
        }
    }

    public void reply(String s) {
//        putText(s);
        pout.println(s);
    }
    
    protected void evalCommand() {
        String newText = textArea.getText().substring(promptCursor);
        promptCursor += newText.length();
        evalCommand(newText);    
    }

    public void runScript(String source) {
        api.setOrb(org.objectweb.openccm.corba.TheORB.getORB());
        if (source != null) {
            putText("source \"" + source + "\"");
            evalCommand("source \"" + source + "\"");
            textArea.append("\n");
            promptCursor -= 2;
        }
    }
    
    // Evaluate the command so far, if possible, printing
    // a continuation prompt if not.
    protected void evalCommand(String newtext) {
//        String newtext = textArea.getText().substring(promptCursor);
//        promptCursor += newtext.length();

        if (commandBuffer.length() > 0) {
            commandBuffer.append("\n");
        }

        commandBuffer.append(newtext);
        String command = commandBuffer.toString();

        if (tclInterp.commandComplete(command)) {
            // Process it
            putText("\n");
            Cursor oldCursor = textArea.getCursor();
            textArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));

            try {
                tclInterp.eval(command);
            } catch (tcl.lang.TclException e) {
            // ignore
            }
            String result = tclInterp.getResult().toString();

            if (result.length() > 0) {
                putText(result + "\n" + mainPrompt);
            } else {
                putText(mainPrompt);
            }

            commandBuffer.setLength(0);
            commandCursor = promptCursor;
            textArea.setCursor(oldCursor);
            updateHistory(command);
        } else {
            putText("\n" + contPrompt);
        }
    }

    // Update the command history
    void updateHistory(String command) {
        historyCursor = 0;

        if (historyCommands.size() == historyLength) {
            historyCommands.removeElementAt(0);
        }

        historyCommands.addElement(command);
    }

    // Replace the command with an entry from the history
    void nextCommand() {
        String text;

        if (historyCursor == 0) {
            text = "";
        } else {
            historyCursor--;
            text = (String)historyCommands.elementAt(historyCommands.size() - historyCursor - 1);
        }

        textArea.replaceRange(text, commandCursor, textArea.getText().length());
    }

    void previousCommand() {
        String text;

        if (historyCursor == historyCommands.size()) {
            return;
        } else {
            historyCursor++;
            text = (String)historyCommands.elementAt(historyCommands.size() - historyCursor);
        }

        textArea.replaceRange(text, commandCursor, textArea.getText().length());
    }



    // The key listener
    class ShellKeyListener extends KeyAdapter  {
        public void keyPressed(KeyEvent keyEvent) {
            // Process keys
            switch ( keyEvent.getKeyCode() ) {
                case KeyEvent.VK_ENTER :
                    keyEvent.consume();
                    evalCommand();
                    break;
                case KeyEvent.VK_BACK_SPACE :

                    if (textArea.getCaretPosition() == promptCursor) {
                        keyEvent.consume();
                    // don't backspace over prompt!
                    }

                    break;
                case KeyEvent.VK_LEFT :

                    if (textArea.getCaretPosition() == promptCursor) {
                        keyEvent.consume();
                    }

                    break;
                case KeyEvent.VK_UP :
                    previousCommand();
                    keyEvent.consume();
                    break;
                case KeyEvent.VK_DOWN :
                    nextCommand();
                    keyEvent.consume();
                    break;
                default :
            // Otherwise we got a regular character. Don't consume it,
            // and TextArea will take care of displaying it.
            }
        }
    }
    
    class TextAreaPrintStream extends java.io.OutputStream  {
        public synchronized void write(int b) {
            // recall that the int should really just be a byte
            b &= 0x000000FF;
            // must convert byte to a char in order to append it
            char c = (char)b;
            putText(String.valueOf(c));
        }

        public synchronized void write(byte[] b, int offset, int length) {
            putText(new String(b, offset, length));
        }
    }

    class TextAreaPrintWriter extends java.io.PrintWriter  {
        
        public TextAreaPrintWriter(java.io.OutputStream out) {
            super(out, true);
        }
   
        public void print(String s) {
            super.print(s);
            try {
                flush();
            } catch (Exception e) {
            }
        }

        public void println(String s) {
            super.println(s);
            try {
                flush();
            } catch (Exception e) {
            } 
        }            
    }
}
