/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.command.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.openccm.command.lib.ReleaseInfo;
import org.objectweb.openccm.corba.TheORB;

/**
 * Abstract base class providing the main structure of
 * a CORBA client application.
 *
 * Subclasses must defined the run() method.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class ApplicationBase
              extends org.objectweb.util.cmdline.lib.ApplicationBase
           implements org.objectweb.openccm.command.api.Application
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ApplicationBase()
    {
        // Calls the org.objectweb.util.cmdline.lib.ApplicationBase constructor.
        super();
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    ApplicationBase(CommandLine commandLine)
    {
        // Calls the org.objectweb.util.cmdline.lib.ApplicationBase constructor.
        super(commandLine);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * Should be defined in subclasses.
     *
     * @return Its identity.
     */
    public String
    getIdentity()
    {
        return getCommandLine().getLabels()[0]
             + ' ' + ReleaseInfo.VERSION;
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Obtains the associated version information.
     *
     * @return The associated version information.
     */
    public String[]
    getVersionInformation()
    {
        String[] result = super.getVersionInformation();
        result[0] = ReleaseInfo.NAME_SHORT + ' '
                  + getIdentity()
                  + " (on top of "
                  + System.getProperty("orb.name", "UNKNOWN")
                  + ' '
                  + System.getProperty("orb.version", "X.X")
                  + ')';
        return result;
    }

    /**
     * Executes the main function, i.e.
     * 1 - Initializes the CORBA::ORB,
     * 2 - Calls super.runMain().
     *
     * @param args The command line arguments.
     */
    public void
    runMain(String[] args)
    {
        try
        {
            // Initializes the CORBA::ORB singleton.
            args = TheORB.initialize(args);
        }
        // If any exception thrown then reports it.
        //
        catch(org.objectweb.util.misc.api.ExceptionWrapper exc)
        {
            report_exception(exc.getException());
        }
        catch(Exception exc)
        {
            report_exception(exc);
        }

        // Calls ApplicationBase.runMain method.
        super.runMain(args);
    }

    /**
     * Starts the application, i.e.
     * 1 - Runs the application,
     * 2 - Destroys the CORBA::ORB.
     *
     * @param args The command line arguments.
     */
    public int
    start(String[] args)
    {
        int status = 0;

        try
        {
            // Runs the application.
            status = run(args);
        }
        finally
        {
           // Destroys the CORBA::ORB singleton.
           TheORB.destroy();
        }

        return status;
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * Note that the CORBA::ORB is always initialized
     * before this method is called.
     *
     * Must be defined in subclasses.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public abstract int
    run(String[] args);
}
