/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Fr?d?ric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;
import org.objectweb.openccm.descriptor.softpkg.ccm.DependencyResolutionFailureException;
import org.objectweb.openccm.Deployment.Server;
import org.objectweb.openccm.Deployment.ServerHelper;
import org.omg.Components.CCMHome;
import org.omg.Components.ConfigValue;
import org.omg.Components.Deployment.ComponentInstallation;
import org.omg.Components.Deployment.Container;

/**
 * This class is the destination deployer implementation able to deploy
 * home on a openccm componentserver.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Fr?d?ric</a>
 *
 * @version 0.1
 */
public class CSDestinationDeployer 
       extends DestinationDeployerBase 
    {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private Server server;
    private Hashtable containerTable;
    private java.util.List config_values;
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    public CSDestinationDeployer(){
        containerTable=new Hashtable();
        config_values=new java.util.LinkedList();
    }
    
    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================
    /**
     * Internal method to retrieve the server which support the component
     * server and component installation.
     * 
     * @return the server describes by destination element
     */
    private Server getServer() 
    throws Exception 
    {
        if (server == null) {           
            server = ServerHelper.narrow(getFindbyDeployer().resolveReference());
        }
        return server;
    }
    /**
     * Internal method to install a home.
     * 
     * @param csConfig
     * @param contConfig
     * @param homeConfig
     * @param implID
     * @param entryPoint
     * @return
     * @throws FatalDeploymentException
     */
    private CCMHome 
    installCCMHome(
           ConfigValue[] csConfig,
           ConfigValue[] contConfig,
           ConfigValue[] homeConfig,
           String implID,
           String entryPoint)
     throws FatalDeploymentException
     {
        
       CCMHome home = null;
        try {
            Container cont = getServer()
                              .provide_component_server()
                              .create_container(contConfig);
            home = cont.install_home(implID, entryPoint, homeConfig);
            containerTable.put(home, cont);
        } catch (Exception e) {
            throw new FatalDeploymentException(this,e,"Cannot deploy the CCMHome on the required destination");
        }
        return home;
     }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    /**
      * Return the component installation of the target host used
      * to install the home.
      * 
      * @return The component installation of the target host
      * @throws FatalDeploymentException if a fatal problem occured during installation
      */  
    public ComponentInstallation 
    getComponentInstallation()
    throws FatalDeploymentException {

        try {
            return getServer().provide_install();
        } catch (Exception e) {
            throw new FatalDeploymentException(this,e,"Cannot acces to the componentinstallation on the required destination");
        }
    }
    /**
     * Make the target destination deployer process all the dependencies
     * of the implementation.
     * 
     * @throws FatalDeploymentException if a problem occured
     *  during processing of dependencies
     */
    public void 
    resolveDependency()
    throws FatalDeploymentException
    {
        try{
            config_values.addAll(Arrays.asList(getSoftpkgDeployer()
                                                .processDependencies
                                                    (getComponentInstallation())));
            List l=Arrays.asList(getImplementationDeployer()
                                 .install(getComponentInstallation()));
              
            config_values.addAll(l);    
                                                    
        }
        catch(DependencyResolutionFailureException e){
            getErrorManager().submitException(e);            
        }           
    }
    /**
     * InstallHome install the home on the precised destination according
     * to the host requirements, such as dependencies and event factory.
     * 
     * @return The home installed
     * @throws FatalDeploymentException
     */    
    public CCMHome 
    installHome()
    throws FatalDeploymentException
    {
      ConfigValue[] config_home =(ConfigValue[])
                                 config_values.toArray(new ConfigValue[config_values.size()]);

       return installCCMHome(new ConfigValue[0],
                       new ConfigValue[0],
                       config_home,
                       getImplementationDeployer().getImplementation().getId(),
                       getImplementationDeployer().getEntryPoint());
    }
    
   
    /**
     * Remove a previously installed CCMHome
     * 
     * @param home the home to remove
     * @throws FatalDeploymentException if a fatal problem occured during uninstallation
     */
     public void 
     removeHome(CCMHome home) 
     throws FatalDeploymentException 
     {
         try {
             Container cont = (Container) containerTable.remove(home);
             cont.remove_home(home);
             cont.remove();
         } catch (Exception e) {
             throw new FatalDeploymentException(e);
         }
     }

}
