/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.menu.lib;

import java.util.List;
import java.util.Vector;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.menu.api.ItemDescription;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.api.MenuElement;

/**
 * Basic implementation of the <code>MenuDescription</code> interface.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicMenuDescription 
  implements MenuDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected boolean inheritTreeMenu_ = true;
    protected boolean inheritTypeMenu_ = true;
    protected List menuElements_ = null;
    protected List childMenuElements_ = null;
    protected String descriptionType_ = Description.MENU_DESCRIPTION;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public BasicMenuDescription(){
        menuElements_ = new Vector();
        childMenuElements_ = new Vector();
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Two <code>BasicMenuDescription</code> are considered as equals 
     * if and only if they have the same items list.
     */
    protected boolean equals(BasicMenuDescription menuDesc){
        if(menuDesc!=null){
            return ExplorerUtils.compareObjects(menuElements_, menuDesc.menuElements_)
                && ExplorerUtils.compareObjects(childMenuElements_, menuDesc.childMenuElements_);
        }
        return false;
    }
    
    // ==================================================================
    //
    // Public methods for Description interface.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#getDescriptionType()
     */
    public String getDescriptionType() {
        return descriptionType_;
    }

    /* (non-Javadoc)
     * @see boolean#isNull()
     */
    public boolean isEmpty() {
        return menuElements_.isEmpty() && childMenuElements_.isEmpty();
    }  
    
    // ==================================================================
    //
    // Public methods for MenuDescription interface.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#setDescriptionType(java.lang.String)
     */
    public void setDescriptionType(String descriptionType) {
        descriptionType_ = descriptionType;
    }    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#setInheritTreeMenu(boolean)
     */
    public void setInheritTreeMenu(boolean inheritTreeMenu) {
        inheritTreeMenu_ = inheritTreeMenu;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#getInheritTreeMenu()
     */
    public boolean getInheritTreeMenu() {
        return inheritTreeMenu_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#setInheritTypeMenu(boolean)
     */
    public void setInheritTypeMenu(boolean inheritTypeMenu) {
        inheritTypeMenu_ = inheritTypeMenu;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#getInheritTypeMenu()
     */
    public boolean getInheritTypeMenu() {
        return inheritTypeMenu_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#addMenuElement(org.objectweb.util.explorer.core.menu.api.MenuElement)
     */
    /**
     * Adds the given element into the elements list if 
     * <ul>
     *   <li>this one is not considered as null.</li>  
     *   <li>this one is not yet present.</li>
     * </ul>
     * The menu elements are separated into two lists: 
     * The first one concerning the current menu itself 
     * and the second one for the children of this menu
     * when it is used in a graphical context.
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#addMenuElement(org.objectweb.util.explorer.core.menu.api.MenuElement)
     */
    public void addMenuElement(MenuElement menuElement) {
        if(menuElement!=null && !menuElement.isEmpty()){
            if(menuElement instanceof ItemDescription){
                ItemDescription itemDesc = (ItemDescription)menuElement;
                List theList = (itemDesc.getTreeChildVisible())?childMenuElements_:menuElements_;
                if(!theList.contains(itemDesc)){
                    theList.add(menuElement);
                }
            } else {
                menuElements_.add(menuElement);
            }
        }
    }

    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#addMenuElements(org.objectweb.util.explorer.core.menu.api.MenuElement[])
     */
    public void addMenuElements(MenuElement[] menuElements) {
        if(menuElements!=null){
            for (int i = 0; i < menuElements.length; i++) {
                addMenuElement(menuElements[i]);
            }
        }

    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#getMenuElements()
     */
    public MenuElement[] getMenuElements() {
        return (MenuElement[])menuElements_.toArray(new MenuElement[menuElements_.size()]);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#getChildrenMenuElements()
     */
    public MenuElement[] getChildrenMenuElements() {
        return (MenuElement[])childMenuElements_.toArray(new MenuElement[childMenuElements_.size()]);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.MenuDescription#getAllMenuElements()
     */
    public MenuElement[] getAllMenuElements() {
        MenuElement[] menuElement = getMenuElements();
        MenuElement[] childrenMenuElement = getChildrenMenuElements();
        boolean needSeparator = menuElement.length>0 && childrenMenuElement.length>0;
        MenuElement[] mergeMenu = new MenuElement[menuElement.length + childrenMenuElement.length + (needSeparator?1:0)];
        System.arraycopy(menuElement,0,mergeMenu,0,menuElement.length);
        if(needSeparator){
            mergeMenu[menuElement.length] = new BasicMenuSeparator();
        }
        System.arraycopy(childrenMenuElement,0,mergeMenu,menuElement.length + (needSeparator?1:0),childrenMenuElement.length);
        return mergeMenu;
    }
    
    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    

    /**
     * Surcharging the Object.equals method.
     * @see Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        if(o!=null && o instanceof BasicMenuDescription){
            return equals((BasicMenuDescription)o);
        }
        return false;
    }
   
    /**
     * Return a String representation of a BasicAcceleratorDescription 
     * @see Object#toString() 
     */    
    public String toString(){
        return "BasicMenuDescription["
            + "inheritTreeMenu=" + inheritTreeMenu_ + ", "
            + "inheritTypeMenu=" + inheritTypeMenu_ + ", "
            + "menuElements=" + ExplorerUtils.arrayToString(menuElements_.toArray())
            + "childMenuElements=" + ExplorerUtils.arrayToString(childMenuElements_.toArray())
            + "]";
    } 
        
}

