/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================
$Id: ExplorerParserLoader.java,v 1.2 2005/07/06 15:36:00 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.parser.lib;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.explorer.core.common.lib.ClassResolver;
import org.objectweb.util.explorer.core.common.lib.XMLFileFilter;
import org.objectweb.util.explorer.explorerConfig.beans.ExplorerBean;
import org.objectweb.util.explorer.explorerConfig.beans.ExplorerBeanImpl;
import org.objectweb.util.explorer.parser.api.ExplorerParser;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class ExplorerParserLoader 
     extends AbstractParser
  implements BindingController
{

    //==================================================================
    //
    // No Internal State.
    //
    // ==================================================================
   
    /** The reference to the dispatcher. */
    protected ExplorerParser dispatcher_;
    
    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal method.
    //
    // ==================================================================   
    
    /**
     * Tries to open an InputStream on the given resource.
     * <ul>
     *   <li>1. First, it looks for the resource in the current class loader.</li>
     *   <li>2. Then, it tries to load the resource as an URL.</li>
     *   <li>3. Finally, it tries to load the resource from within the file system.</li>
     * </ul>
     * @param resource The resource for which an InputStream needs to be opened.
     * @return An InputStream opened on the given resource (or null).
     */
    protected InputStream getInputStream(String resource){
        // Tries to load the resource from the ClassLoader
        InputStream inputStream = ClassResolver.getResourceAsStream(resource);
        if(inputStream==null) {
            try {
                // Tries to load the resource as an URL
                inputStream = new URL(resource).openStream();
            } catch (MalformedURLException e) {
                try {
                    // Tries to load the resource on the file system
                    inputStream = new FileInputStream(resource);
                } catch (FileNotFoundException e1) {
                    getTrace().info(resource + ": File not found on the file system!");
                    return null;
                }
            } catch (IOException e) {
                getTrace().info(resource + ": Failed to open stream on resource!");
                return null;
            } 
        } 
        return inputStream;        
    }
   
    /**
     * Parses and interprets the given stream.
     */
    protected void parse(InputStream inputStream){
        if(inputStream!=null){
            try {
                // We can't validate with DTD at this time (no way to fix the DTD file)
                ExplorerBean explorer = ExplorerBeanImpl.unmarshalBean(new InputStreamReader(inputStream),false);
                if (explorer != null){
                    dispatcher_.parseExplorer(explorer);
                }
            } catch (java.io.IOException e) {
                getTrace().info("Error during parsing!\n" + e.getMessage());
            }
        }
    }
    
    /**
     * Parses and interprets the given resource.
     * If the given resource is a local directory, all its contained files are interpreted. 
     */
    protected void parse(String resource){
        // Checks if the given resource is a directory
        File file = new File(resource);
        if (file.exists() && file.isDirectory()) {
            File[] files = file.listFiles(new XMLFileFilter());
            for (int i = 0; i < files.length; i++) {
                parse(files[i].toString());
            }
        } else {
            parse(getInputStream(resource));
        }
    }
    
    // ==================================================================
    //
    // Public methods for BindingController interface.
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String itfName, Object itfValue)
            throws NoSuchInterfaceException, IllegalBindingException,
            IllegalLifeCycleException {
        if(itfName.equals(ExplorerParser.EXPLORER_PARSER))
            dispatcher_ = (ExplorerParser)itfValue;
    }
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String itfName) throws NoSuchInterfaceException {
        if(itfName.equals(ExplorerParser.EXPLORER_PARSER))
            return dispatcher_;
        return null;
    }
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String itfName) throws NoSuchInterfaceException,
            IllegalBindingException, IllegalLifeCycleException {
        if(itfName.equals(ExplorerParser.EXPLORER_PARSER))
            dispatcher_ = null;
    }    
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc(){
        return new String[]{ExplorerParser.EXPLORER_PARSER};
    }
    
    // ==================================================================
    //
    // Public method inherits from AbstractParser class.
    //
    // ==================================================================    
    // Functional concerns.
    
    /**
     * Parses and interprets the defined Explorer XML files. 
     */
    public void parse() {
        // The use of an iterator here is impossible because the list
        // may grow and such an iterator will throw 
        // the java.util.ConcurrentModificationException
        int i = 0;
        while(i<urlFiles_.size()){
            parse((String)urlFiles_.get(i));
            i++;
        }
        urlFiles_.clear();
    }

}
