/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================
$Id: DefaultDialogBox.java,v 1.2 2005/07/06 15:36:00 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.swing.gui.lib;

/** The Explorer API's imports */
import java.awt.Dimension;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;

import org.objectweb.util.explorer.swing.gui.api.DialogAction;
import org.objectweb.util.explorer.swing.gui.api.DialogBox;
import org.objectweb.util.explorer.swing.gui.api.ElementBox;
import org.objectweb.util.explorer.swing.gui.api.ValidateReport;


/**
 * Basic implementation of a DialogBox. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class DefaultDialogBox
    extends JDialog 
    implements DialogBox
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** Indicates if it's the first show method invocation. */
    protected boolean firstShowInvocation_ = true;
    
    /** The OK and Cancel buttons */
    protected Action validateAction_, cancelAction_;
    
    /** The actions to execute */
    protected DialogAction validateDialogAction_, cancelDialogAction_;

    /** The composite Box */
    protected Box compositeBox_;
    
    /** The list of ElementBox */
    protected List elements_;

    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    /**
     * Empty constructor
     */
    public DefaultDialogBox(){
        this("");
    }

    /**
     * Constructs a DialogBox with a title
     * @param title
     */
    public DefaultDialogBox(String title){
        super();
        
        elements_ = new Vector();
        
        setModal(true);
        setTitle(title);
      
        getContentPane().add(createMainBox());
    }

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    /**
     * Creates a composite Box: a box which contains the ElementBox
     * @return The GUI box. 
     */
    protected Box createCompositeBox(){
        Box b = Box.createHorizontalBox();
        b.add(Box.createHorizontalStrut(10));
        compositeBox_ = Box.createVerticalBox();
        compositeBox_.add(Box.createVerticalStrut(10));
        b.add(compositeBox_);
        b.add(Box.createHorizontalStrut(10));
        return b;
    }

    /**
     * Creates the main box: A box and two buttons.
     * @return The GUI box.
     */
    protected Box createMainBox(){
        Box mainBox = Box.createVerticalBox();
        mainBox.add(createCompositeBox());
        
        Box buttonBox = Box.createHorizontalBox();
        buttonBox.add(Box.createHorizontalGlue());
        validateAction_ = new DefaultOKAction();
        buttonBox.add(new JButton(validateAction_));
        buttonBox.add(Box.createHorizontalStrut(10));
        cancelAction_ = new DefaultCancelAction();
        buttonBox.add(new JButton(cancelAction_));
        buttonBox.add(Box.createHorizontalGlue());
        
        mainBox.add(buttonBox);
        mainBox.add(Box.createVerticalStrut(10));
        
        return mainBox;
    }

    //==================================================================
    //
    // Public methods for DialogBox interface.
    //
    //==================================================================

    /**
     * Adds an element into the current JDialog without being concerned about the display.
     * @param box The element to add.
     */
    public void addElementBox(ElementBox box){
        elements_.add(box);
        compositeBox_.add(box.getBox());
        compositeBox_.add(Box.createVerticalStrut(10));
    }
    
    /**
     * Returns the JDialog contains by the DialogBox.
     * @return The JDialog contains by the DialogBox.
     */
    public JDialog getJDialog(){
        return this;
    }

    /**
     * Shows the dialog box
     */
    public void show(){
        // Initializes the elements
        Iterator it = elements_.iterator();
        while (it.hasNext()) {
            ElementBox element = (ElementBox) it.next();
            element.preInitialize();
        }
        // Packs the window.
        pack();
        // Initialize the windows at the first time
        if(firstShowInvocation_){
            firstShowInvocation_ = false;
            setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
            setResizable(false); 
            pack();

            Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2);
        }
        // Displays the window
        super.show();
    }
    
    /**
     * Hides the dialog box
     */
    public void hide(){
        // Post initializes the elements
        Iterator it = elements_.iterator();
        while (it.hasNext()) {
            ElementBox element = (ElementBox) it.next();
            element.postInitialize();
        }
        // Hides the window
        super.hide();
    }
    
    /**
     * Fixes the action to use for the validate button
     * @param action The action to use.
     */ 
    public void setValidateAction(DialogAction action){
        validateDialogAction_ = action;    
    }

    /**
     * Fixes the label of the validate button (default value is "OK").
     * @param label The label of the validate button
     */
    public void setValidateLabel(String label){
        validateAction_.putValue(Action.NAME, label);
    }
    
    /**
     * Fixes the action to use for the cancel button
     * @param action The action to use.
     */
    public void setCancelAction(DialogAction action){
        cancelDialogAction_ = action;
    }
    
    /**
     * Fixes the label of the cancel button (default value is "Cancel").
     * @param label The label of the cancel button
     */
    public void setCancelLabel(String label){
        cancelAction_.putValue(Action.NAME, label);
    }

    /**
     * Validates all the elements contains by the JDialog.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateDialog(){
        Iterator it = elements_.iterator();
        while (it.hasNext()) {
            ElementBox element = (ElementBox) it.next();
            ValidateReport report = element.validateBox();
            if(!report.getResult()){
                return report;
            }
        }
        return new DefaultValidateReport();         
    }
    
    //==================================================================
    //
    // Internal classes.
    //
    //==================================================================
    
    /**
     * Default action for OK button
     */
    protected class DefaultOKAction extends AbstractAction{

        /**
         * Default constructor
         */
        protected DefaultOKAction() {
            super("OK", null);
        }

        /**
         * Action to execute on click
         */
        public void actionPerformed(java.awt.event.ActionEvent ae) {
            ValidateReport report = validateDialog();            
            if(report.getResult()){
                boolean exception = false; 
                if(DefaultDialogBox.this.validateDialogAction_!=null){
                    try{
                        DefaultDialogBox.this.validateDialogAction_.executeAction();
                    }catch(Exception e){
                        exception = true;
                        JOptionPane.showMessageDialog(null, e.getClass().getName() + ":\n" + e.getMessage(), "Exception (" + ae.getActionCommand() + ")", JOptionPane.ERROR_MESSAGE);
                    }
                }
                if(!exception)
                    //DefaultDialogBox.this.setVisible(false);
                    hide();
            }else
                JOptionPane.showMessageDialog(null, report.getMessage(), "Params error !", JOptionPane.ERROR_MESSAGE);
        }

    }    
    
    /**
     * Default action for Cancel button
     */
    protected class DefaultCancelAction extends AbstractAction {

        /**
         * Default constructor
         */
        protected DefaultCancelAction() {
            super("Cancel", null);
        }

        /**
         * Action to execute on click
         */
        public void actionPerformed(java.awt.event.ActionEvent ae) {
            boolean exception = false; 
            if(DefaultDialogBox.this.cancelDialogAction_!=null){
                try{
                    DefaultDialogBox.this.cancelDialogAction_.executeAction();
                }catch(Exception e){
                    exception = true;
                    JOptionPane.showMessageDialog(null, e.getMessage(), ae.getActionCommand(), JOptionPane.ERROR_MESSAGE);
                }
            } else
                //DefaultDialogBox.this.setVisible(false);
                hide();
        }

    }

}