/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): Romain Rouvoy.

--------------------------------------------------------------------
$Id: XBootClassLoader.java,v 1.1 2004/05/19 15:58:29 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher;


import java.net.MalformedURLException ;
import java.net.URLClassLoader ;
import java.net.URL ;

import org.objectweb.util.trace.TraceSystem;


/**
 * Specific URLClassLoader {@link URLClassLoader}.<BR>
 * <p>
 * This loader overwrite URLClassLoader dafult behaviour. <BR>
 * It searches requested classes first in locally registered URLs and
 * then in the parent class loader.
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * @version 0.2
 */
public class XBootClassLoader
     extends URLClassLoader
{
    // ==================================================================
    //
    // Constructors
    //
    // ==================================================================

    /**
     * Default constructor.
     */
    public XBootClassLoader() { 
        super(new URL[0]); 
    }

    /**
     * Constructor with parent ClassLoader.
     *
     * @param parent - the parent classloader to use.
     */
    public XBootClassLoader(ClassLoader parent) { 
        super(new URL[0], parent); 
    }

    // ==================================================================
    //
    // Internal methods
    //
    // ==================================================================

    /**
     * Loads the Class with the specified name.
     *
     * @param name The name of the Class to be loaded (must not be null).
     * @param resolve if true then resolve the class.
     *
     * @return the requested Class object.
     * @see Class
     *
     * @throws ClassNotFoundException if the requested class does not
     *                                exist in the loader's classpath.
     */
    protected Class loadClass(String name, boolean resolve)
        throws ClassNotFoundException
    { // check if the class has already been loaded
        Class c = findLoadedClass(name);
        if (c == null)
            try { // If not found, call findClass for finding the class
                c = findClass(name);
            } catch (ClassNotFoundException e) {
                return super.loadClass(name, resolve);
            }
        if (resolve)
            resolveClass(c);
        return c;
    }

    // ==================================================================
    //
    // Public methods
    //
    // ==================================================================

    /**
     * Searches and loads a Class in the classpath of this ClassLoader.
     * @param name - The name of the class to be loaded (must not be null).
     * @return The required Class object.
     * @see Class
     * @throws ClassNotFoundException if the requested class does not 
     *                                exist in the loader's classpath.
     */
    public Class findClass(String name)
        throws ClassNotFoundException
    {
        return super.findClass(name);
    }

    /**
     * Loads the Class with the specified name.
     * @param name - The name of the class to be loaded (must not be null).
     * @return the required Class object.
     * @see Class
     * @throws ClassNotFoundException if the requested class does not 
     *                                exist in the loader's classpath.
     */
    public Class loadClass(String name)
        throws ClassNotFoundException
    {
        TraceSystem.get("classloader").debug("Loading class "+name);
        return loadClass(name, false);
    }
    
    /**
     * Adds URL to the ClassLoader.
     * @param url - A valid URL to load (must not be null).
     */
    public void addURL(URL url) 
    { // This class is overriden for making it public
        TraceSystem.get("classloader").info("Registering URL "+url);
        // Check if the archive exist
        try {
            url.openConnection().connect();
        } catch (Exception ex) {
            throw new LauncherException(ex);
        }
        if (!java.util.Arrays.asList(getURLs()).contains(url))
            super.addURL(url);
    }


    /**
     * Adds URL to the ClassLoader.
     * @param url - a string representation of an URL (must not be null).
     * @throws MalformedURLException the syntax of the url is invalid.
     */
    public void addURL(String url)
        throws MalformedURLException
    {
        this.addURL(new URL(url));
    }


    /**
     * Adds a set of URLs to the ClassLoader.
     * @param urls - a string representation of a set of URL (must not be null).
     * @throws MalformedURLException the syntax of an url is invalid.
     */
    public void addURL(String[] urls)
        throws MalformedURLException
    {
        for (int i=0 ; i < urls.length ; i++)
            this.addURL(urls[i]);
    }

    
    /**
     * Checks if to classloaders are equivalent.
     * Two classloaders are equivalent if they reference the same URLs.
     * @param obj the classloader to compare to the current one.
     * @return true if classloaders are equivalent, false in other cases.
     */
    public boolean equals(Object obj)
    {
        if (obj instanceof XBootClassLoader) {
            XBootClassLoader loader = (XBootClassLoader) obj;
            return java.util.Arrays.equals(getURLs(), loader.getURLs());
        }
        return false ;
    }
}
