
#include <microccm_CarCharacteristics_executor.h>

#include <cmath>

#include <threading_utils.h>

#include <iostream>
using namespace std;

class CarCharacteristics_impl : public CCM_CarCharacteristics,
				public CCM_CarCharacteristicsInterface,
				public CCM_SpeedInterface,
				public Components::SessionComponent
{
public:

  ~CarCharacteristics_impl();

  CCM_CarCharacteristicsInterface_ptr get_car_caracs();
  CCM_SpeedInterface_ptr get_speed_indic();

  void configuration_complete() throw( Components::InvalidConfiguration );
  void set_session_context( Components::SessionContext_ptr ctx )
    throw( Components::CCMException );

  // ---- CarCharacteristicsInterface

  CORBA::Float throttle_factor();
  void throttle_factor( CORBA::Float );

  CORBA::Float mass();
  void mass( CORBA::Float );

  CORBA::Float friction_factor();
  void friction_factor( CORBA::Float );

  // ---- SpeedInterface

  CORBA::Float speed();
  void speed( CORBA::Float );

private:

  CCM_CarCharacteristics_Context_ptr ctx_;

  pthread_t the_thread_;

  threading::protected_resource< CORBA::Float > throttle_factor_;
  threading::protected_resource< CORBA::Float > mass_;
  threading::protected_resource< CORBA::Float > friction_factor_;

  threading::protected_resource< CORBA::Float > current_speed_;

  friend void* CarCharacteristics_thread_func( void* );
};

// ---------------------------------------------------------------------

CarCharacteristics_impl::~CarCharacteristics_impl()
{
  pthread_cancel( the_thread_ );
}

CCM_CarCharacteristicsInterface_ptr CarCharacteristics_impl::get_car_caracs()
{
  return this;
}

CCM_SpeedInterface_ptr CarCharacteristics_impl::get_speed_indic()
{
  return this;
}

void CarCharacteristics_impl::configuration_complete() 
  throw( Components::InvalidConfiguration )
{
  pthread_create( &the_thread_, NULL, CarCharacteristics_thread_func, this );
}

void CarCharacteristics_impl::set_session_context( Components::SessionContext_ptr ctx ) throw( Components::CCMException )
{
  ctx_ = CCM_CarCharacteristics_Context::_narrow( ctx );
}

// ----

CORBA::Float CarCharacteristics_impl::throttle_factor()
{
  return throttle_factor_;
}

void CarCharacteristics_impl::throttle_factor( CORBA::Float value )
{
  //  cerr << "Setting throttle_factor to be " << value << endl;
  throttle_factor_ = value;
}

CORBA::Float CarCharacteristics_impl::mass()
{
  return mass_;
}

void CarCharacteristics_impl::mass( CORBA::Float value )
{
  mass_ = value;
}

CORBA::Float CarCharacteristics_impl::friction_factor()
{
  return friction_factor_;
}

void CarCharacteristics_impl::friction_factor( CORBA::Float value )
{
  friction_factor_ = value;
}

// -----

CORBA::Float CarCharacteristics_impl::speed()
{
  return current_speed_ ;
}

void CarCharacteristics_impl::speed( CORBA::Float f )
{
  current_speed_ = f;
}

// -----

void* CarCharacteristics_thread_func( void* s )
{
  CarCharacteristics_impl* ci = ( CarCharacteristics_impl* ) s;  

  float time_step = 50e-3;

  for( ; ; )
    {
      usleep( lrint( 1e6 * time_step ) ); // sleeps 50 ms

      // -- compute numerical solution of O.D.E --

      CORBA::Float throttle_value
	= ci->ctx_->get_connection_throttle_information()->throttle_value();

      CORBA::Float m = ci->mass_;
      CORBA::Float current_speed = ci->current_speed_;
      CORBA::Float throttle_fact = ci->throttle_factor_;
      CORBA::Float fric_fact = ci->friction_factor_;

      float Delta
	= m*m + 4*fric_fact*time_step * ( m*current_speed + throttle_fact* throttle_value * time_step );

//       cerr << " CarCharacteristics ::  Delta == " << Delta << " time_step == " << time_step 
// 	   << " mass == " << m << " throttle_fact == " << throttle_fact 
// 	   << " throttle_value == " << throttle_value << " fricfact == " << fric_fact << endl;

      ci->current_speed_ = ( -m + sqrt( Delta ) ) / ( 2 * fric_fact * time_step );
    }

  return NULL;
}


// ---------------- Home -------------------------------------------------


class CarCharacteristicsHome_impl : public CCM_CarCharacteristicsHome
{
public:

  Components::EnterpriseComponent_ptr create( ) throw( Components::CCMException );
};

Components::EnterpriseComponent_ptr CarCharacteristicsHome_impl::create()
  throw( Components::CCMException )
{
  return new CarCharacteristics_impl;
}

extern "C"
{
  Components::HomeExecutorBase* create_home_exec()
  {
    return new CarCharacteristicsHome_impl;
  }
}

// -----------------------------------------------------------------------

