/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): Fr?d?ric Briclet, Philippe Merle.

====================================================================*/

package org.objectweb.ccm.deploytool;

// Package dependencies
import java.io.File;
import java.net.MalformedURLException;
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.util.cmdline.lib.DefaultOptionFlag;
import org.omg.Components.CreateFailure;

/**
 * This is the CCM XML deployment application.
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 * @version 0.1
 */

public class DeploymentApplication
     extends ApplicationBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the deployer. */
    //private DeploymentProcess _deployer;
    /*The name of the zip file to deploy*/
    private String zipFile ;
    /*Boolean denoting if the deployment must be distributed or not*/
    private boolean distributedDeployment =false;
    /*In distributed deployment case the assembly factory name to use*/
    private String assemblyFactoryName =null;
    /*The assembly factory information*/
    private DefaultOptionArgument factory_info;
    /*This flag indicate that the deployment must be done locally*/
    private  DefaultOptionFlag local_flag; 
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    DeploymentApplication()
    {
        super(
            new DefaultCommandLine(
                 new String[]{"ccm_deploy","ccm_deploy"}
                ,/*new String[]{"-Fassembly_factory_name",*/"zip_file.aar"/*}*/
                ,new String[]{"The component assembly archive to deploy"}
                ,true
                )
            );
        factory_info=new DefaultOptionArgument(
                                "-F",
                                "factory_name",
                                "The name of the factory to use, default is DefaultFactory ",
                                "DefaultFactory");
        local_flag=new DefaultOptionFlag("-l",
                                        "This flag indicates that the deployment must be "
                                        +"done locally without using an assembly factory",
                                        false);
         
               
        getCommandLine().addOption(local_flag);                                  
        getCommandLine().addOption(factory_info);  
    }


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    private class UnsupportedArchiveFormat extends Exception{
        public UnsupportedArchiveFormat(String format){
            super("The archive format ["+format+
                  "] is not supported by the deployment.");
        }
    }
    
    private void
    computeArgs(String args[])
    {
        for(int i=0;i<args.length;i++){
            if(args[i].length()>3&&(
               args[i].endsWith("aar")||args[i].endsWith("zip")))
                this.zipFile=args[i];
            /*else  if(args[i].equals("-F"))
                this.distributedDeployment=true;
            else  if(args[i].startsWith("-F")){
                this.assemblyFactoryName=
                    System.getProperty("Assembly_factory_ns_dir")+args[i].substring(2);        
                this.distributedDeployment=true;
            }*/
        } 
        
        if(local_flag.isSet()){
            distributedDeployment=false;
            return;
        }else distributedDeployment=true;
           
    
       if(factory_info.isSet()){
           assemblyFactoryName=
                          System.getProperty("Assembly_factory_ns_dir")+factory_info.getArgument();      
            getConsole().message("Use the specified assembly factory ["+factory_info.getArgument()+"]");
       }
    }   
      
    private void
    proceedLocalDeployment() 
    throws CreateFailure 
    {
        getConsole().message("Local deployment processing");
        try{        
            File f=new File(zipFile);
           (new org.objectweb.openccm.Deployment.AssemblyImpl(f.toURL())).build();
        }
        catch(MalformedURLException e){
            //System.err.println("MalFormedURL:");
            throw new CreateFailure();
        }
    }
    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * @return Its identity.
     */
    public String
    getIdentity()
    {
        return "ccm_deploy"
             + ' ' + org.objectweb.ccm.util.ReleaseInfo.VERSION_MAJOR
             + '.' + org.objectweb.ccm.util.ReleaseInfo.VERSION_MINOR;
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Executes the main function, i.e.
     * 1 - Initializes the CORBA::ORB,
     * 2 - Calls super.runMain().
     *
     * @param args The command line arguments.
     */
    public void
    runMain(String[] args)
    {
        try
        {
            // Initializes the CORBA::ORB singleton.
            // Initializes the CORBA::ORB singleton.
            args=org.objectweb.openccm.corba.TheORB.initialize(args);
            org.objectweb.openccm.Components.Runtime.init(org.objectweb.openccm.corba.TheORB.getORB());
            // Set EntityResolver for CCM DTDs
            org.objectweb.ccm.descriptor.EntityResolver.setCCMResolver();
        }
        // If any exception thrown then reports it.
        //
        catch(org.objectweb.util.api.ExceptionWrapper exc)
        {
            report_exception(exc.getException());
        }
        catch(Exception exc)
        { 
            report_exception(exc);
        }

        // Calls ApplicationBase.runMain method.
        super.runMain(args);
    }

    /**
     * Starts the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    start(String[] args)
    {
        
        computeArgs(args);
        boolean result = deploy(zipFile);
        return result?0:-1;
    }

    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================


    /**
     * Deploys a XML Component Assembly Descriptor.
     *
     * @param assembly_filename The XML Component Assembly Descriptor file name.
     *
     * @return True if the deployment successes, otherwise false.
     */
    public boolean
    deploy(String zip_file)
    {
        boolean result=true;
            
        getConsole()
            .message(
                "Initiating deployment sequence of "
                + zip_file
                );
        
        org.omg.Components.Deployment.Assembly ass;
        
        try{

            try{
                
                if(distributedDeployment){
                    org.omg.Components.Deployment.AssemblyFactory af ;
                        
                    if(assemblyFactoryName!=null){
                        
                        org.omg.CosNaming.NamingContextExt context=
                            org.omg.CosNaming.NamingContextExtHelper
                            .narrow(org.objectweb.openccm.corba.TheNameService
                                    .getNamingContext()
                                    .getNamingContext());
                        
                        
                        af= org.objectweb.openccm
                            .OpenCCM_DCI.AssemblyFactoryManagerHelper
                            .narrow(context.resolve(context.to_name(assemblyFactoryName)))
                            .provide_assembly_factory();       
                    }                              
                    else {
                        
                        org.objectweb.openccm.OpenCCM_DCI.DCIManager
                            DCI=
                            org.objectweb.openccm.
                            OpenCCM_DCI.DCIManagerHelper.
                            narrow(org.objectweb.openccm.corba
                                   .TheORB.resolve_initial_reference("DCIManager"));
                        af=DCI.get_connection_assembly_factory();
                        
                    }
                    
                    if(af==null)
                        throw new org.omg.CosNaming.NamingContextPackage.NotFound();
                        
                    //Add the use of an Http server to provide the assemblyfile
                    java.net.ServerSocket s = new java.net.ServerSocket(0); 
                    
                    String archive_url = "http://"
                                + java.net.InetAddress.getLocalHost().getHostAddress()
                                + ':' + s.getLocalPort() + '/' +zip_file;
                    getConsole().message("Archive URL " + archive_url);        

                    //Thread serverThread=
                    (new Thread(new MicroServerHttp(s))).start();

                    org.omg.Components.Cookie cok=
                        af.create_assembly(archive_url);
                    
                    ass=af.lookup(cok);
                    ass.build();  
                    //ass.setConsole(getConsole());  
                }
                else{
                    getConsole().message("Proceed a local deployment, ");
                    proceedLocalDeployment(); 
                }
            }
            catch(java.io.IOException ioe)
              {
                    getConsole().error
                        ("Cannot use assembly factory to download file, proceed a local deployment, ");
                    getConsole().error
                        ("the tear_down function will be disable from console");
                  
                   proceedLocalDeployment(); 
                   
                    
                }  
            /*catch(java.net.UnknownHostException uoe)
              {
                    getConsole().error
                        ("Cannot use assembly factory to download file, process a local deployment, ");
                    getConsole().error
                        ("the tear_down function will be disable from console");
                  
                    (new org.objectweb.openccm.Deployment.AssemblyImpl(assembly_filename))
                         .build();
                   
                    
                         }  */
            catch( org.omg.CosNaming.NamingContextPackage.NotFound nf) 
                {
		    getConsole().error
                        ("Cannot found a valid assembly factory, proceed a local deployment,");
		    getConsole().error
                        ("the tear_down function will be disable from console");
                  
                   proceedLocalDeployment();                   
                    
                }  
            catch(org.omg.CosNaming.NamingContextPackage.CannotProceed cp)
                {
		    getConsole().error
                        ("Cannot use name service to found a valid assembly factory, proceed a local deployment,");
		    getConsole().error
                        ("the tear_down function will be disable from console");                  
                   proceedLocalDeployment();               
                }  
            catch(org.omg.CosNaming.NamingContextPackage.InvalidName in)
                {
		    getConsole().error
                        ("Cannot found a valid assembly factory, proceed a local deployment,");
		    getConsole().error
                        ("the tear_down function will be disable from console");                  
                   proceedLocalDeployment(); 
                    
                }  
            catch(org.omg.Components.Deployment.InvalidAssembly IA)
                {
		    getConsole().error
                        ("Internal error occured during retrieving of assembly, cannot process deployment");                
                    return false;
                }          
         
            //ass.build();      

        }
        catch(org.omg.Components.Deployment.InvalidLocation il)
            {
                getConsole().error
                    ("Cannot found the CAD file, check your path and restart the deployment");
                result=false;
            }
        catch(org.omg.Components.CreateFailure cf)
            {
                getConsole().error
                    ("Cannot achieved assembly deployment, be carefull only a partial assembly");
                getConsole().error
                    ("is deployed, assembly behavior may be corrupt");
                result=false;
            }    
                    
        if (!result)
        {
            getConsole()
                .error(
                    "Sequence aborted during deployment of "
                    + zip_file
                    );
            return false;
        }
       
        getConsole()
            .message(
                "Successfully deployed "
                + zip_file
                );

        return result;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        DeploymentApplication application = new DeploymentApplication();
        application.runMain(args);
    }
}
