/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): _______________________________.

====================================================================*/

package org.objectweb.ccm.runtime.cif.lib;

import org.omg.Components.ExecutorSegmentBase;

import org.objectweb.ccm.runtime.cif.api.SegmentHome;

import java.util.HashMap;

/**
 * Abstract base class for all component segment classes generated from CIDL.
 *
 * This implements the CORBA Components ExecutorLocator Strategy.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public abstract class ComponentSegmentBase
              extends org.objectweb.corba.util.LocalObjectBase
           implements org.objectweb.ccm.runtime.cif.api.ComponentSegment,
                      org.objectweb.ccm.runtime.cif.api.Segment
{
    // ==================================================================
    //
    // Internal State.
    //
    // ==================================================================

    /**
     * Table of all executor segments composing a component implementation.
     */
    ExecutorSegmentBase[] executor_segments_;

    /** Reference to the executor segment home. */
    SegmentHome segment_home_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ComponentSegmentBase()
    {
        // Inits internal state.
        executor_segments_ = null;
        segment_home_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.ccm.runtime.cif.api.Segment
    //
    // ==================================================================

    /**
     * Set the main segment executor (segid = 0).
     *
     * @param seg - The main segment.
     */
    public void
    set_main_segment(org.objectweb.ccm.runtime.cif.api.ComponentSegment seg)
    {
        // Nothing to do!
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.ccm.runtime.cif.api.ComponentSegment
    //
    // ==================================================================

    /**
     * Sets the number of executor segments.
     *
     * This method is called by the constructor of
     * component segment classes generated from CIDL.
     *
     * @param nb_segments The number of executor segments
     *                    for this CIDL component executor.
     */
    public void
    set_nb_segments(int nb_segments)
    {
        // Allocates the table for all executor segments.
        executor_segments_ = new ExecutorSegmentBase[nb_segments];

        // Sets the component segment.
        executor_segments_[0] = this;
    }

    /**
     * Sets the associated segment home.
     *
     * @param sh The associated segment home.
     */
    public void
    set_segment_home(SegmentHome sh)
    {
        segment_home_ = sh;
    }

    /**
     * Gets an executor segment.
     *
     * If the requested executor segment does not exist
     * then it is created by the segment home.
     *
     * @param segid The executor segment id.
     *
     * @return The executor segment.
     */
    public ExecutorSegmentBase
    get_executor_segment(int segid)
    {
        // Gets the executor segment in the table.
        ExecutorSegmentBase result = executor_segments_[segid];

        // If no executor segment
        if(result == null)
        {
            // Uses the executor segment home to create it.
            result = segment_home_.create_executor_segment(segid);
            ((org.objectweb.ccm.runtime.cif.api.Segment)result).set_main_segment(this);

            // Stores the executor segment in the table.
            executor_segments_[segid] = result;
        }

        // Returns the executor segment.
        return result;
    }

    /**
     * Obtains the segmentation table.
     *
     * This method is implemented by component segment classes generated
     * from CIDL according to segmentation described in CIDL compositions.
     *
     * @return A HashMap containing <facet_name, segid> associations.
     */
    abstract public java.util.HashMap
    get_segmentation_table();

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::EnterpriseComponent
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/EnterpriseComponent/configuration_complete:1.0
    //
    /**
     * Complete the component configuration.
     *
     * Default empty implementation that could be
     * overridden by user component executor implementation.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        getLogger().trace(this,
            "Completing configuration: Do nothing by default");

        // Do nothing by default!
    }

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::ExecutorLocator
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/ExecutorLocator/obtain_executor:1.0
    //
    /**
     * Obtain an executor.
     *
     * @param name The facet name.
     *
     * @return The executor reference.
     *
     * @exception org.omg.Components.CCMException Thrown if any error.
     */
    public org.omg.CORBA.Object
    obtain_executor(java.lang.String name)
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this, "Obtaining executor " + name);

        // Obtains the segmentation table.
        HashMap segmentation_table = get_segmentation_table();

        // Gets the Integer associated to name.
        Integer segid = (Integer)segmentation_table.get(name);

        // If no associated segment identifier
        if(segid == null)
        {
            getLogger().error(this,
               "No executor segment associated to facet " + name);

            throw new org.omg.Components.CCMException();
        }

        // Returns the executor segment associated to segid.
        return get_executor_segment(segid.intValue());
    }

    //
    //  IDL:omg.org/Components/ExecutorLocator/release_executor:1.0
    //
    /**
     * Release an executor.
     *
     * @param executor The executor reference.
     *
     * @exception org.omg.Components.CCMException Thrown if any error.
     */
    public void
    release_executor(org.omg.CORBA.Object executor)
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this, "Releasing executor " + executor +
                                ": Do nothing by default");

        // Do nothing by default!
    }
}
