/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.dependencies.lib;

// Package dependencies.
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.CompositionDecl;
import org.objectweb.openccm.ast.api.ComponentDecl;
import org.objectweb.openccm.ast.api.HomeExecutorDecl;
import org.objectweb.openccm.ast.api.ExecutorDecl;
import org.objectweb.openccm.ast.api.SegmentDecl;
import org.objectweb.openccm.ast.utils.api.CompositionInfo;
import java.util.Set;


/**
 * This class allows you to get classes dependencies for a CIDL declaration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class CIDLDeclarationDependencies
     extends IDL3DeclarationDependencies
  implements org.objectweb.corba.generator.dependencies.api.CIDLDeclarationDependencies
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    CIDLDeclarationDependencies()
    {
        // Init internal state
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.generator.cidl.api.CIDLDeclarationDependencies
    //
    // ==================================================================

    /**
     * Get dependencies for a CIDL declaration.
     *
     * @param decl - Get dependencies for this declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getDeclarationDependencies(Declaration decl)
    {
        Set files = null;

        files = new java.util.HashSet();

        if (decl.getDeclKind() == DeclarationKind.dk_composition)
        {
            files.addAll( getCompositionDependencies((CompositionDecl) decl) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_home_executor)
        {
            files.addAll( getHomeExecutorDependencies((HomeExecutorDecl) decl) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_executor)
        {
            files.addAll( getExecutorDependencies((ExecutorDecl) decl) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_proxy_home)
        {
            // TO DO
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_segment)
        {
            files.addAll( getSegmentDependencies((SegmentDecl) decl) );
        }
        else
        {
            // Call to IDL3 dependencies
            files.addAll( super.getDeclarationDependencies(decl, new java.util.HashSet()) );
        }
            // TO DO
            /* PSDL declaration kinds :
               | dk_abstract_storage_home | dk_abstract_storage_type
               | dk_psdl_operation
               | dk_storage_home | dk_storage_home_factory
               | dk_storage_home_key | dk_storage_type
               | dk_storage_type_state_member
               | dk_storage_type_store_directive;
            */

        return files;
    }

   /**
     * Get Composition File dependencies.
     *
     * @param composition - The composition declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getCompositionDependencies(CompositionDecl composition)
    {
        CompositionInfo comp = null;
        Set files = null;

        files = new java.util.TreeSet();
        comp = new org.objectweb.openccm.ast.utils.lib.CompositionInfo(composition);

        // Add home executor dependencies
        files.addAll( getHomeExecutorDependencies(comp.getHomeExecutor()) );

        // Add component executor dependencies
        files.addAll( getExecutorDependencies(comp.getComponentExecutor()) );

        return files;
    }

   /**
     * Get Home Executor File dependencies.
     *
     * @param home_exec - The home executor declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getHomeExecutorDependencies(HomeExecutorDecl home_exec)
    {
        Set files = null;
        String path = null;

        path = _translator.getAsDirectory(home_exec);
        files = new java.util.TreeSet();

        // Home Executor CIF Interfaces
        addStantardIdlFiles(files, path + "CIF_" + home_exec.getName());
        addOperationsClass(files, path + "CIF_" + home_exec.getName());

        // Home Executor skeleton class
        addObjectClass(files, path + home_exec.getName());

        // Add Home dependencies
        files.addAll( getHomeDependencies( home_exec.getHomeType(),
                                           new java.util.HashSet() )
                    );

        return files;
    }

    /**
     * Get Component Executor File dependencies.
     *
     * @param exec - The Component executor declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getExecutorDependencies(ExecutorDecl exec)
    {
        Set files = null;
        String path = null;
        ComponentDecl comp = null;
        HomeExecutorDecl home_exec = null;
        Declaration[] decls = null;

        path = _translator.getAsDirectory(exec.getParent());
        files = new java.util.TreeSet();

        // Component Executor local Interfaces
        addStantardIdlFiles(files, path + "CIF_" + exec.getName());
        addOperationsClass(files, path + "CIF_" + exec.getName());

        // Component Executor Segment Base CIF Interfaces
        addStantardIdlFiles(files, path + "CIF_SegmentBase");
        addOperationsClass(files, path + "CIF_SegmentBase");

        // Component Executor skeleton class
        addObjectClass(files, path + exec.getName());

        // Add Component dependencies
        home_exec = (HomeExecutorDecl) exec.getParent();
        comp = home_exec.getHomeType().getManagedComponent();
        files.addAll( getComponentDependencies( comp,
                                                new java.util.HashSet() )
                    );

        // Add segments dependencies
        decls = exec.getContents(true, DeclarationKind.dk_segment);
        for (int i=0; i<decls.length; i++)
        {
            files.addAll( getSegmentDependencies((SegmentDecl) decls[i]) );
        }

        return files;
    }

     /**
     * Get Segment File dependencies.
     *
     * @param seg - The Segment declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getSegmentDependencies(SegmentDecl seg)
    {
        Set files = null;
        String path = null;

        path = _translator.getAsDirectory(seg.getParent().getParent());
        files = new java.util.TreeSet();

        // Segment CIF Interfaces
        addStantardIdlFiles(files, path + "CIF_" + seg.getName());
        addOperationsClass(files, path + "CIF_" + seg.getName());

        // Segment skeleton class
        addObjectClass(files, path + seg.getName());

        return files;
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
