/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.dependencies.lib;

// Package dependencies.
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.CompositionDecl;
import org.objectweb.openccm.ast.api.HomeExecutorDecl;
import org.objectweb.openccm.ast.api.ExecutorDecl;
import org.objectweb.openccm.ast.api.Scope;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import java.util.Set;
import java.util.TreeSet;
import java.util.Iterator;
import java.io.File;
import java.io.FileWriter;

/**
 * This class allows you to generate class file dependencies
 * for a CIDL Composition.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class CIDLFileDependenciesResolver
     extends org.objectweb.openccm.generator.common.lib.GeneratorBase
  implements org.objectweb.corba.generator.dependencies.api.CIDLFileDependenciesResolver
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Delimiter constant */
    private static final String delimiter =
    "==================================================================\n";

    /** Class to resolve CIDL declarations dependencies. */
    org.objectweb.corba.generator.dependencies.api.CIDLDeclarationDependencies _resolver;

    /** Filters to apply to dependencies.
        A dependencie that starts with <filter> */
    String[] _filters;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */
    public
    CIDLFileDependenciesResolver(org.objectweb.openccm.ast.api.AST ast)
    {
        super(ast);

        // Init internal states
        _resolver = new org.objectweb.corba.generator.dependencies.lib.CIDLDeclarationDependencies();
        _filters = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Apply filters to input.
     * If input starts with one of filters, it won't be added.
     *
     * @param input - The string to check.
     *
     * @return input if filters don't match, else an empty string.
     */
    private String
    applyFilers(String input)
    {
        if (_filters == null)
            return input;

        for (int i=0; i<_filters.length; i++)
        {
            if ( input.startsWith(_filters[i]) )
                return "";
        }

        return input;
    }

    /**
     * Add Component Executor File dependencies to output.
     *
     * @param output - The output stream to write in.
     * @param exec - The Component executor declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    private void
    addExecutorDependencies( FileWriter output,
                             ExecutorDecl exec )
    throws java.io.IOException
    {
        Set files = null;
        Iterator it = null;

        files = _resolver.getExecutorDependencies(exec);
        it = (new TreeSet(files)).iterator();

        output.write("# " + delimiter);
        output.write("# Component " + exec.getAbsoluteName() + "\n");
        output.write("# " + delimiter);

        while ( it.hasNext() )
        {
            output.write( applyFilers(it.next() + "\n") );
        }
        output.write("\n");
    }

    /**
     * Add Home ExecutorFile dependencies to output.
     *
     * @param output - The output stream to write in.
     * @param home_exec - The home declaration.
     *
     * @throws java.io.IOException if a I/O error occurs.
     */
    private void
    addHomeExecutorDependencies( FileWriter output,
                                 HomeExecutorDecl home_exec )
    throws java.io.IOException
    {
        Set files = null;
        Iterator it = null;

        files = _resolver.getHomeExecutorDependencies(home_exec);
        it = (new TreeSet(files)).iterator();

        output.write("# " + delimiter);
        output.write("# Home " + home_exec.getAbsoluteName() + "\n");
        output.write("# " + delimiter);

        while ( it.hasNext() )
        {
            output.write( applyFilers(it.next() + "\n") );
        }
        output.write("\n");
    }

    /**
     * Add Composition File dependencies to output.
     *
     * @param output - The output stream to write in.
     * @param composition - The composition declaration.
     *
     * @throws java.io.IOException if a I/O error occurs.
     */
    private void
    addCompositionDependencies(FileWriter output,
                               CompositionDecl composition)
    throws java.io.IOException
    {
        org.objectweb.openccm.ast.utils.api.CompositionInfo comp = null;

        comp = new org.objectweb.openccm.ast.utils.lib.CompositionInfo(composition);

        output.write("# " + delimiter);
        output.write("# Package for Composition :\n");
        output.write("# " + composition.getAbsoluteName() + "\n");
        output.write("# " + delimiter);
        output.write("\n");

        // Add component executor dependencies
        addExecutorDependencies(output, comp.getComponentExecutor());

        // Add home executor dependencies
        addHomeExecutorDependencies(output, comp.getHomeExecutor());
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.generator.cidl.api.CIDLFileDependenciesResolver
    //
    // ==================================================================

    /**
     * Set filters for class dependencies.
     *
     * @param filters - The list of filters to add.
     */
    public void
    set_filters(String[] filters)
    {
        _filters = filters;
    }

    /**
     * Generate class files dependencies for a composition.
     *
     * @param composition - The composition declaration.
     * @param file - The name of the file to write dependencies in.
     *
     * @throws GenerationException if an exception occurs.
     */
    public void
    generate_composition_dependencies(CompositionDecl composition,
                                      String file)
    {
        java.util.Iterator it = null;
        File output_file = null;
        FileWriter output = null;
        int i = 0;

        try
        {
            // Check output file name
            output_file = new File(file);
            while ( output_file.exists() )
            {
                output_file = new File( file + String.valueOf(i) );
                i++;
            }
            // Create the file and its directory tree
            int index = output_file.getPath().lastIndexOf( System.getProperty("file.separator") );
            if (index != -1)
            {
                File dir = new File( output_file.getPath().substring(0, index+1) );
                dir.mkdirs();
            }
            output_file.createNewFile();

            // Add dependencies to this file
            output = new FileWriter(output_file);
            addCompositionDependencies(output, composition);
            output.close();
            System.out.println("File '" + output_file.getAbsolutePath() + "' has been generated");
        }catch(java.io.IOException ex){
            ex.printStackTrace();
        }
    }

    /**
     * Generate class files dependencies for a composition.
     *
     * @param name - The name of the composition.
     * @param file - The name of the file to write dependencies in.
     *
     * @throws GenerationException if an exception occurs.
     */
    public void
    generate_composition_dependencies(String name, String file)
    throws GenerationException
    {
        CompositionDecl composition = null;

        // Get the composition declaration
        composition = (CompositionDecl) getDeclaration(name);
        generate_composition_dependencies(composition, file);
    }

    /**
     * Generate class files dependencies for all composition
     * contained in this scope.
     *
     * @param filescope - The source AST FileScope returned by the compile() method.
     * @param base_dir - The base directory for dependencies output.
     *
     * @throws GenerationException if an exception occurs.
     */
    public void
    generate_dependencies(Scope scope,
                          String base_dir)
    throws GenerationException
    {
        java.util.List vect = null;
        CompositionDecl[] compositions = null;
        String filename = null;

        vect = getAllDeclarations(scope, DeclarationKind.dk_composition);
        compositions = (CompositionDecl[])vect.toArray(new CompositionDecl[0]);
        for (int i=0; i<compositions.length; i++)
        {
            filename = base_dir + "/" + compositions[i].getName() + ".dep";
            generate_composition_dependencies(compositions[i], filename);
        }
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
