/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Philippe Merle.

====================================================================
$Id: OpenCCMTask.java,v 1.4 2005/06/07 15:58:08 merle Exp $
====================================================================*/

package org.objectweb.openccm.task;

//Package dependencies
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Delete;

/**
 * This task run an OpenCCM application build process.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class OpenCCMTask
     extends Task
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** Ordered list of Runnable elements. */
    protected List runnable_elements_;

    /** The OpenCCM Interface Repository. */
    protected IR3Start ir3_start_;

    /** To disable OMG IDL2.x mappings. */
    private boolean no_mappings_;

    /** OpenCCM status before running the task */
    private boolean ccm_installed;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * The default constructor.
     */
    public 
    OpenCCMTask()
    {
        // Init internal states
        runnable_elements_ = new ArrayList();
        ir3_start_ = null;
        ccm_installed = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    
    /**
     * Create a directory.
     * 
     * @param dir - The full directory name. 
     */
    private void
    create_directory(String dir)
    {
        new File(dir).mkdir();
    }
    
    /**
     * Install the OpenCCM platform. 
     */
    private void
    ccm_install(String config_dir)
    {
        if ( new File(config_dir).exists() )
        {
            ccm_installed = true ;
            System.out.println("An OpenCCM Platform has been found in "+config_dir+", we'll use it for code generation purpose.");
        }
        else
        {
            System.out.println("An OpenCCM Platform will be temporary installed in "+config_dir+" for code generation purpose.");

            System.out.println("Creating the " + config_dir + " directory.");
            create_directory(config_dir);

            System.out.println("OpenCCM Platform is installed.");
        }
    }

    /**
     * Deinstall the OpenCCM platform. 
     */
    private void
    ccm_deinstall(String config_dir)
    {
        // de-install the platform if it wasn't running
        if  ( !ccm_installed )
        {
            Delete delete_task = new Delete();

            delete_task.setProject( getProject() );
            delete_task.setTaskName("ccm_deinstall");
            delete_task.setDir(new File(config_dir));
            delete_task.execute();
        }
    }
    
    /**
     * Run the Interface Repository.
     */
    private void
    ir3_start(String config_dir)
    {
        if ( new File(
            getProject().getProperty("OpenCCM_CONFIG_DIR")
            +System.getProperty("file.separator")
            +"IR3.IOR")
            .exists() )
        {
            System.out.println("An Interface Repository has been found, we'll use it for code generation purpose.");
        }
        else
        {
            System.out.println("An Interface Repository will be temporary started for code generation purpose.");

            // Run the IR
            ir3_start_ = new IR3Start( getProject().getProperties() );
            ir3_start_.setNo_mappings(no_mappings_);
            ir3_start_.run(true);

            System.out.println("Interface Repository is started.");
        }
    }

    /**
     * Stop the Interface Repository. 
     */
    private void
    ir3_stop()
    {
        // stop the IR if it wasn't running
        if( ir3_start_ != null )
        {
            // Stop the IR
            ir3_start_.stop();
        }
    }

    /**
     * Run subtasks elements.
     * 
     * @throws BuildException if an error occurs.
     */
    private void
    run_elements()
    throws BuildException
    {
        for (Iterator it = runnable_elements_.iterator(); it.hasNext();)
        {
            ((Runnable) it.next()).run(); 
        }
    }
    
    /**
     * Run OpenCCM build process.
     * 
     * @throws BuildException if an error occurs.
     */
    private void
    run_openccm_build()
    throws BuildException
    {
        IR3Feed ir3_feed = null;
        String idl_dir = null;
        
        ir3_start( getProject().getProperty("OpenCCM_CONFIG_DIR") );
        
        // Get the OpenCCM idl dir
        idl_dir = getProject().getProperty("OpenCCM_idl_dir");
        
        // Run user commands
        run_elements();

    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * To disable OMG IDL2.x mappings.
     *
     * @param b - If true, disables OMG IDL2.x mappings.
     */
    public void
    setNo_mappings(boolean b)
    {
        this.no_mappings_ = b;
    }

    /**
     * The method executing the task.
     *
     * @throws BuildException if an error occurs.
     */
    public void
    execute()
    throws BuildException
    {
        // Install the OpenCCM platform
        ccm_install( getProject().getProperty("OpenCCM_CONFIG_DIR") );

        try
        {
            run_openccm_build();
        }catch (BuildException e){
            //Stop the IR and deinstall the platform
            ir3_stop();
            ccm_deinstall( getProject().getProperty("OpenCCM_CONFIG_DIR") );
            throw new BuildException("OpenCCM build failed!");  
        }

        // Stop the IR.
        ir3_stop();

        // Deinstall the OpenCCM platform
        ccm_deinstall( getProject().getProperty("OpenCCM_CONFIG_DIR") );  
    }
    
    /**
     * Create an inner element for ir3_feed.
     */
    public IR3Feed
    createIr3_feed()
    {
        IR3Feed ir3_feed = null;
        
        ir3_feed = new IR3Feed( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(ir3_feed);
                
        return ir3_feed;
    }
    
    /**
     * Create an inner element for ir3_idl3.
     */
    public IR3toIDL3
    createIr3_idl3()
    {
        IR3toIDL3 ir3_idl3 = null;
        
        ir3_idl3 = new IR3toIDL3( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(ir3_idl3);
                
        return ir3_idl3;
    }

    /**
     * Create an inner element for ir3_idl2.
     */
    public IR3toIDL2
    createIr3_idl2()
    {
        IR3toIDL2 ir3_idl2 = null;
        
        ir3_idl2 = new IR3toIDL2( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(ir3_idl2);
                
        return ir3_idl2;
    }

    /**
     * Create an inner element for psdl_java.
     */
    public PSDLtoPSDL
    createPsdl()
    {
        PSDLtoPSDL psdl_psdl = null;
        
        psdl_psdl = new PSDLtoPSDL( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(psdl_psdl);
                
        return psdl_psdl;
    }

    /**
     * Create an inner element for psdl_java.
     */
    public CIDLtoCIDL
    createCidl()
    {
        CIDLtoCIDL cidl_cidl = null;
        
        cidl_cidl = new CIDLtoCIDL( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(cidl_cidl);
                
        return cidl_cidl;
    }

    /**
     * Create an inner element for psdl_java.
     */
    public PsdlToJava
    createPsdl_java()
    {
        PsdlToJava psdl_java = null;
        
        psdl_java = new PsdlToJava( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(psdl_java);
                
        return psdl_java;
    }

    /**
     * Create an inner element for ir3_java.
     */
    public IR3toJava
    createIr3_java()
    {
        IR3toJava ir3_java = null;
        
        ir3_java = new IR3toJava( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(ir3_java);
                
        return ir3_java;
    }

    /**
     * Create an inner element for cidl_cif.
     */
    public CIDLtoCIF
    createCidl_cif()
    {
        CIDLtoCIF cidl_cif = null;
        
        cidl_cif = new CIDLtoCIF( getProject().getProperties() );
        // Add to ordered list
        runnable_elements_.add(cidl_cif);
                
        return cidl_cif;
    }
}
