/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.OptionArgument; 
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.openccm.ast.api.FileScope;

/**
 * This class allows you to generate Java mapping from a PSDL file.
 *
 * @author <a href="mailto:Chrsitophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class PSDLtoJavaCommand
     extends org.objectweb.openccm.command.lib.CompilerGeneratorBase
  implements org.objectweb.openccm.pss.generator.common.api.PSDLtoJavaCommand
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Output directory Option. */
    OptionArgument odir_; // -d <directory>

    /** User properties file name. */
    OptionArgument user_props_; // -userprop <file_name>

    /** Persistent Framework used. */
    OptionArgument backend_; // -backend [jdo,hibernate]

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    PSDLtoJavaCommand()
    {
        // Calls the CompilerGeneratorBase constructor.
        super(new DefaultCommandLine("psdl_java",
                                     "file",
                                     "Compile an OMG PSDL file and generate its associated Java mapping",
                                     true));

        // Specify output base directory
        odir_ = new DefaultOptionArgument(
                        new String[] { "-d" },
                        "outputdir",
                        new String[] {
                          "Generate Java mapping into directory <outputdir>,",
                          "default is generated/"
                        },
                        "generated" );
        getCommandLine().addOption(odir_);

        // Specify User properties file name
        user_props_ = new DefaultOptionArgument(
                        new String[] { "-userprop" },
                        "user_properties_file",
                        new String[] {
                          "Get user classes definition from the properties file <user_properties_file>",
                          "default is \"user.properties\""
                        },
                        "user.properties" );
        getCommandLine().addOption(user_props_);

        // Specify Persistent Framework name
        backend_ = new DefaultOptionArgument(
                        new String[] { "-backend" },
                        "jdo|hibernate",
                        new String[] {
                          "Choose the persistent framework to use : jdo or hibernate",
                          "default is \"jdo\""
                        },
                        "jdo" );
        getCommandLine().addOption(backend_);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        String filename = args[0];

        // Inits the OpenCCM Abstract Syntax Tree.
        if(!initAST())
            return -1;

        // Compiles the file.
        FileScope fileScope = compile(filename);
        if(fileScope == null)
            return -1;

        // Generates PSDL Mapping
        if (!generate_java_mapping(fileScope,
                                   filename, // Input File
                                   getOutputDirectory(), // Base directory for Java Mapping
                                   getUserPropertiesFile(), // User JDO properties file
                                   getBackend() )) // The Persistent framework
            return -1;

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Generator
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.command.api.PSDLtoJavaCommand
    //
    // ==================================================================

    /**
     * Obtains the output directory.
     *
     * @result The output directory.
     */
    public String
    getOutputDirectory()
    {
        return odir_.getArgument();
    }

    /**
     * Obtains the user properties file name.
     *
     * @result The user properties file name.
     */
    public String
    getUserPropertiesFile()
    {
        return user_props_.getArgument();
    }

    /**
     * Obtains the persistent framework choosen.
     *
     * @result The persistent framework choosen.
     */
    public String
    getBackend()
    {
        return backend_.getArgument();
    }

    /**
     * Generates PSDL Mapping from an AST FileScope instance.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope The source AST FileScope returned by the compile() method.
     * @param inputfile - The source file.
     * @param base_dir - The base directory for java mapping.
     * @param user_properties_file - The user jdo properties file.
     * @param backend - The target persistent framework.
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_java_mapping(FileScope filescope,
                          String inputfile,
                          String base_dir,
                          String user_properties_file,
                          String backend)
    {
        // Launchs the PSDL to Java generator.
        org.objectweb.openccm.pss.generator.common.api.PSDL2JavaGenerator gen =
            new org.objectweb.openccm.pss.generator.common.lib.PSDL2JavaGenerator(getAST());

        // Generates Java Mapping.
        getConsole().message("Generating Java mapping from " + inputfile + " ...");
        try
        {
            gen.psdl_to_java( base_dir,
                              filescope,
                              user_properties_file,
                              backend);
        }
        catch(org.objectweb.openccm.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }
        getConsole().message("Java mapping generated.");

        return true;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        PSDLtoJavaCommand psdl2java = new PSDLtoJavaCommand();
        psdl2java.runMain(args);
    }
}
