/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.lib;

// Package dependencies
/** To access AST. */
import org.objectweb.openccm.ast.api.StorageTypeDecl;
import org.objectweb.openccm.ast.api.AbstractStorageTypeDecl;
import org.objectweb.openccm.ast.api.StorageTypeStateMemberDecl;
import org.objectweb.openccm.ast.api.PsdlOperationDecl;
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.Declaration;
/** To access Java AST. */
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
/** Others. */
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.generator.common.api.GeneratorBase;
import org.objectweb.openccm.pss.generator.common.api.PSDL2JavaGenerator;
import org.objectweb.openccm.pss.generator.common.api.StateMemberMapping;

import java.util.List;


/**
 * This class generates Storage Type Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */

public abstract class StorageTypeMapping
           implements org.objectweb.openccm.pss.generator.common.api.StorageTypeMapping
{

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The storage type declaration. */
    private StorageTypeDecl st_;

    /** All state members to map. */
    private java.util.List all_states_;

    /** Read only state members. */
    private java.util.List read_states_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StorageTypeMapping()
    {
        // Init internal state
        st_ = null;
        read_states_ = new java.util.ArrayList();
        all_states_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Has this abstract storage type an operation declaration ?
     * (including implemented abstract storage types)
     *
     * @param ast - The abstract storage type to test.
     *
     * @return True if an operation is contained, else false.
     **/
    private boolean
    hasOperation(AbstractStorageTypeDecl ast)
    {
        Declaration[] decls = null;
        decls = ast.getContents(true, DeclarationKind.dk_psdl_operation);
        if (decls.length > 0)
        {
            return true;
        }
        else
        {
            // Check implemented abstract storage types
            AbstractStorageTypeDecl[] implemented = null;

            implemented = ast.getAbstractStorageTypeList().getAbstractStorageTypes();
            for (int i=0; i<implemented.length; i++)
            {
                if ( hasOperation(implemented[i]) )
                    return true;
            }
            return false;
        }
    }

    /**
     * Map state members.
     *
     * @param state_members - State Mebers to map.
     * @param clazz - The abstract storage type mapping class.
     * @param ct - The abstract storage type constructor mapping.
     * @param generator - The PSDL to Java generator.
     **/
    private void
    mapStateMembers(StorageTypeStateMemberDecl[] state_members,
                    ClassObject clazz,
                    ConstructorObject ct,
                    PSDL2JavaGenerator generator)
    {
        for (int i=0; i<state_members.length; i++)
        {
            StateMemberMapping state = null;
            
            state = (StateMemberMapping) MappingObjectFactory.getClass("StateMemberMapping"); 
            state.setStateMember(state_members[i]);

            state.toJavaAccessors(generator.getTranslator(), clazz, true);
            state.toJavaAttribute(generator.getTranslator(), clazz, st_);
            if ( state_members[i].isReadonly() )
            {
                state.toJavaParameter(generator.getTranslator(), ct);
                read_states_.add(state_members[i].getName());
                state.toJavaReadOnlyStateMemberModifier(generator.getTranslator(), clazz);
            }
       }
    }

    /**
     * Add Instance Callbacks methods to this storage type.
     *
     * @param generator - The PSDL to Java generator.
     * @param clazz - The abstract storage type mapping class.
     **/
    abstract protected void
    addInstanceCallbacks( PSDL2JavaGenerator generator,
                          ClassObject clazz );

    /**
     * Get The base class for JDO Storage Type Mapping.
     * 
     * @return The base class name.
     */
    abstract protected String
    getStorageTypeBaseClass();

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
   /** 
    * Set the Storage Type Declaration to map. 
    *
    * @param st - The Storage Type Declaration to map.
    */
    public void
    setStorageType(StorageTypeDecl st)
    {
        st_ = st;
    }

    /**
     * Get the list of all state members that will be mapped.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members.
     **/
    public java.util.List
    getAllStates(GeneratorBase generator)
    {
        AbstractStorageTypeDecl[] impl_directly = null;;
        List vect = null;

        if (all_states_ == null)
        {
            all_states_ = new java.util.ArrayList();

            // Add State Members implemented directly
            impl_directly = st_.getDirectlyImplementedAbstractStorageTypes();
            for (int j=0; j<impl_directly.length; j++)
            {
                try{
                    vect = generator.getDeclarations(impl_directly[j], DeclarationKind.dk_storage_type_state_member);
                }catch(org.objectweb.openccm.generator.common.lib.GenerationException ex){
                    ex.printStackTrace();
                }
                all_states_.addAll( vect );
            }

            // Add own State Members
            try{
                vect = generator.getDeclarations(st_, DeclarationKind.dk_storage_type_state_member);
            }catch(org.objectweb.openccm.generator.common.lib.GenerationException ex){
                ex.printStackTrace();
            }
            all_states_.addAll( vect );
        }
        return all_states_;
    }

    /**
     * Get the list of all state members that will be mapped to native types.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to native types.
     **/
    public java.util.List
    getStatesToNative(GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_native = null;
        java.util.Iterator it = null;

        to_native = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if (mapping_type == null)
            {
                to_native.add(state);
            }
        }
        return to_native;
    }

    /**
     * Get the list of all state members that will be mapped to an IOR.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to an IOR.
     **/
    public java.util.List
    getStatesToIOR(GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_ior = null;
        java.util.Iterator it = null;

        to_ior = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if ( (mapping_type != null) &&
                 (mapping_type.compareTo("java.lang.String") == 0) )
            {
                to_ior.add(state);
            }
        }
        return to_ior;
    }

    /**
     * Get the list of all state members that will be mapped to a byte array.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to a byte array.
     **/
    public java.util.List
    getStatesToCodec(GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_codec = null;
        java.util.Iterator it = null;

        to_codec = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if ( (mapping_type != null) &&
                 (mapping_type.compareTo("byte[]") == 0) )
            {
                to_codec.add(state);
            }
        }
        return to_codec;
    }

    /**
     * Get the list of all state members that will be mapped to a byte array.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to a byte array.
     **/
    public java.util.List
    getStatesToClass(GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_class = null;
        java.util.Iterator it = null;

        to_class = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if ( (mapping_type != null) &&
                 (mapping_type.compareTo("java.lang.String") != 0) &&
                 (mapping_type.compareTo("byte[]") != 0) )
            {
                to_class.add(state);
            }
        }
        return to_class;
    }

    /**
     * Generate a java persistent capable class for this storage type.
     *
     * @param storage_object - If true, generates mapping for a storage object.
     * @param generator - The PSDL to Java generator.
     */
    public void
    toJava( boolean storage_object,
            PSDL2JavaGenerator generator )
    throws GenerationException
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        MethodObject method = null;
        AttributeObject att = null;
        ParameterObject param = null;
        AbstractStorageTypeDecl[] implemented = null,
                                  impl_directly = null;;
        StorageTypeDecl base = null;
        List vect = null;
        StorageTypeStateMemberDecl[] state_members = null;
        PsdlOperationDecl[] operations = null;

        read_states_ = new java.util.ArrayList();

        clazz = new ClassObjectImpl(st_.getName());
        clazz.addComment(" Storage Type "+st_.getName());
        clazz.addComment("");
        clazz.addComment(" @author OpenCCM PSDL Compiler");
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(generator.getTranslator().getPackage(st_));

        // Add Implemented Abstract Storage Type by st
        implemented = st_.getAbstractStorageTypeList().getAbstractStorageTypes();
        for (int i=0; i<implemented.length; i++)
        {
            clazz.addImplementedObject(generator.getTranslator().getAbsoluteName(implemented[i]));
        }

        // Check if any of the abstract storagetypes implemented declares an operation
        int i = 0;
        boolean found = false;
        clazz.setAbstract(false);
        while ((i<implemented.length) && !found)
        {
            if (hasOperation(implemented[i]))
            {
                clazz.setAbstract(true);
                found = true;
            }
            i++;
        }

        // Add Inherited Storage Type if there is one
        base = st_.getBaseStorageType();
        if (base != null)
            clazz.addInheritedObject(generator.getTranslator().getAbsoluteName(base));
        else if (storage_object)
            clazz.addInheritedObject( getStorageTypeBaseClass() );

        // Add the default constructor
        ct = new ConstructorObjectImpl();
        ct.getImpl().setMacro("ST_CONSTRUCTOR");
        clazz.addConstructor(ct);

        if (storage_object)
        {
            // Add the sh_rid attribute
            att = new AttributeObjectImpl();
            att.addComment("The Storage Home Repository ID.");
            att.setName("sh_rid");
            att.setType("String");
            att.setModifier( ModifierKindImpl.mk_private );
            clazz.addAttribute(att);

            // Override the initialize method
            method = new MethodObjectImpl();
            method.addComment("Initialize the storage object.");
            method.setName("initialize");
            method.setReturnType("void");
            param = new ParameterObjectImpl();
            param.setName("storage_home");
            param.setType("org.omg.CosPersistentState.StorageHomeBase");
            method.addParameter(param);
            param = new ParameterObjectImpl();
            param.setName("pid");
            param.setType("byte[]");
            method.addParameter(param);
            method.getImpl().setMacro("ST_INITIALIZE_METHOD");
            clazz.addMethod(method);
        }

        java.util.List all_states = getAllStates(generator);
        state_members = (StorageTypeStateMemberDecl[])all_states.toArray(new StorageTypeStateMemberDecl[0]);
        mapStateMembers(state_members, clazz, ct, generator);

        // Complete the storage type constructor
        ct.getImpl().addContextValue("read_states", read_states_);
        // Add store directives
        vect = generator.getDeclarations(st_, DeclarationKind.dk_storage_type_store_directive);
        ct.getImpl().addContextValue("store_directives", vect);
        ct.getImpl().addContextValue("translator", generator.getTranslator());

        addInstanceCallbacks(generator, clazz);

        generator.getJavaRepository().addObject(clazz);
    }
}
