/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Alex Andrushchak.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.pss.runtime.hibernate.lib;

import net.sf.hibernate.cfg.Configuration;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.Session;

/**
 * This is a class to connect an hibernate implementation.
 *
 * @author <a href="mailto:pretend@ukr.net">Alex Andrushchak</a>
 *
 * @version 0.1
 */

public class Connector
    extends org.objectweb.openccm.pss.runtime.common.lib.ConnectorBase
    implements org.objectweb.openccm.pss.runtime.hibernate.api.Connector
{
    // ==================================================================
    //
    // Constants.
    //
    // ==================================================================

    public static final String NAME = "org.objectweb.openccm.pss.runtime.Hibernate";

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Hibernate Configuration. */
    private Configuration _configuration = null;
    
    /** Hibernate Session Factory. */
    private SessionFactory _session_factory = null;

    /** Configuration properties. */
    java.util.Properties props_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public Connector()
    {
        super(NAME);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Load Hibernate configuration file.
     * 
     * @param file - The configuration file to load.
     */
    private void
    load_hibernate_properties(String file)
    {
        props_ = new java.util.Properties();

        try {
            java.io.InputStream propStream = null;
    
            propStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(file);
            props_.load(propStream);
        }catch (Exception e) {
            throw new RuntimeException("Cannot load " + file + " file" + e);
        }

        System.getProperties().putAll(props_);
        getConfiguration().setProperties(props_);
    }

    /**
     * Configure the Hibernate connector.
     * 
     * @return An hibernate configuration instance.
     */
    protected Configuration
    getConfiguration()
    {
        if ( _configuration == null )
        {
            _configuration = new Configuration();
            _configuration.setProperties(props_);
        }
        
        return _configuration;
    }

    /**
     * Register a persistent class to an Hibernate configuration.
     * This registration allows Hibernate to find associated hbm.xml files.
     * 
     * @param clazz - The class to register.
     */
    protected void
    registerPersistentClass(Class clazz)
    {
        if ( (clazz != null) &&
             (clazz.getName().compareTo("org.objectweb.openccm.pss.runtime.hibernate.lib.StorageObjectBase") != 0) )
        {
            registerPersistentClass( clazz.getSuperclass() );
            try {
                getConfiguration().addClass(clazz);
            } catch (net.sf.hibernate.MappingException ex) {
                // Configuration file not found
                // Nothing to do as mapping file may not be needeed
            }
        }
    }
    
    /**
     * Get an hibernate Session Factory
     * 
     * @return an hibernate Session Factory.
     */
    protected SessionFactory
    getSessionFactory()
    {
        if ( null == _session_factory ) {
            try {
                _session_factory = getConfiguration().buildSessionFactory();
            } catch (net.sf.hibernate.HibernateException ex) {
                throw new RuntimeException("Cannot instantiate hibernate session factory", ex);
            }
        }
        return _session_factory;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Connector
    //
    // ==================================================================

    /**
     * Create a basic session.
     */
    public org.omg.CosPersistentState.Session
    create_basic_session(short access_mode,
                         org.omg.CosPersistentState.Parameter[] additional_parameters)
    {
        String db_url = null,
               db_driver = null,
               user_name = null,
               password = null;
        java.sql.Connection connection = null;

        // Load Hibernate configuration properties
        if (additional_parameters.length == 1)
        {
            org.omg.CosPersistentState.Parameter param = additional_parameters[0];

            if (param.name.compareTo(Connector.CONF_FILE_KEY) == 0)
            {
                load_hibernate_properties( param.val.extract_string() );
            }
            else
                throw new RuntimeException("CONF_FILE parameter is missing!");
        }
        else
            throw new RuntimeException("CONF_FILE parameter is missing!");

       db_driver = props_.getProperty("hibernate.connection.driver_class");
       db_url = props_.getProperty("hibernate.connection.url");
       user_name = props_.getProperty("hibernate.connection.username");
       password = props_.getProperty("hibernate.connection.password");
       // System.out.println(db_driver);
       // System.out.println(db_url);
       // System.out.println(user_name);
       // System.out.println(password);

        // JDBC driver loading
        try {
            Class.forName(db_driver);
        } catch ( ClassNotFoundException ex ) {
            throw new RuntimeException(ex);
        }

        // Create database connection
        try {

            if ( (user_name == null) || (password == null) ) {
                connection = java.sql.DriverManager.getConnection(db_url);
            } else {
                connection = java.sql.DriverManager.getConnection(db_url, user_name, password);
            }
        } catch ( java.sql.SQLException ex ) {
            throw new RuntimeException(ex);
        }

        // Create Hibernate session
        Session session = getSessionFactory().openSession(connection);
        return new org.objectweb.openccm.pss.runtime.hibernate.lib.Session(get_free_id(), session);
/*
        try {
            Session session = getSessionFactory().openSession();

            return new org.objectweb.openccm.pss.runtime.hibernate.lib.Session(get_free_id(), session);
        } catch (net.sf.hibernate.HibernateException ex) {
            throw new RuntimeException("Cannot create hibernate session", ex);
        }
*/
    }

    /**
     * Create a transactional session.
     */
    public org.omg.CosPersistentState.TransactionalSession
    create_transactional_session(short access_mode,
                                 short default_isolation_level,
                                 org.omg.CosPersistentState.EndOfAssociationCallback callback,
                                 org.omg.CosPersistentState.Parameter[] additional_parameters)
    {
        // TO DO
        return null;
    }

    /**
     * Create a session pool.
     */
    public org.omg.CosPersistentState.SessionPool
    create_session_pool(short access_mode,
                        short tx_policy,
                        org.omg.CosPersistentState.Parameter[] additional_parameters)
    {
        // TO DO
        return null;
    }

    /**
     * Get the current session.
     */
    public org.omg.CosPersistentState.TransactionalSession
    current_session()
    {
        // TO DO
        return null;
    }

    /**
     * Get sessions.
     */
    public org.omg.CosPersistentState.TransactionalSession[]
    sessions(org.omg.CosTransactions.Coordinator transaction)
    {
        // TO DO
        return null;
    }


    // ==================================================================
    //
    // Methods for org.objectweb.openccm.services.pss.api.Connector
    //
    // ==================================================================

    /**
     * Register a storage object factory.
     */
    public java.lang.Class
    register_storage_object_factory(String storage_type_name,
                                    java.lang.Class storage_object_factory)
    {
        registerPersistentClass(storage_object_factory);
        return super.register_storage_object_factory(storage_type_name, storage_object_factory);
    }

}
