/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: UML2CCM.java,v 1.1 2004/05/26 11:25:34 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.lib;

import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.model.ModelContext;
import ispuml.mdaTransformation.rules.PropertyValues;

import java.io.File;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.jmi.reflect.RefPackage;

import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.openccm.ast.api.Scope;
import org.objectweb.openccm.uml.transformation.ast.CCMASTModelContext;
import org.objectweb.openccm.uml.transformation.ast.CCMASTModelCreateUtils;
import org.objectweb.openccm.uml.transformation.engine.MappingRuleEngineWithCollection;
import org.objectweb.openccm.uml.transformation.log.ConsoleSingleton;
import org.objectweb.openccm.uml.transformation.modfact.JmiModFactModelCreateUtils;
import org.objectweb.openccm.uml.transformation.modfact.JmiModfactUML14ModelContext;

/**
 * Implementation of the uml2ccm command.
 *
 * @author <a href="mailto:Pierre.Carpentier@lifl.fr">Pierre Carpentier</a>
 *
 * @version 1.0
 */

public class UML2CCM
  extends org.objectweb.openccm.command.lib.CompilerBase
    implements org.objectweb.openccm.uml.api.UML2CCM
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The UML XMI file importer. */
    private org.omg.uml.xmi.UmlXMIImport xmiImport;
    
    /** Flag to indicate the kind of the Java implementation (monolithic or CIF). */
    private boolean monolithic = false;
    
    /** The output directory for the generated files. */
    private String outputDirectory = "generated";

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    UML2CCM()
    {
        // Calls the CompilerBase constructor.
        super(new DefaultCommandLine("uml2ccm",
                                     "xmi_uml_document",
                                     "Transform the XMI 1.1 UML 1.4 document to the OMG IDL 3.0, OMG CIDL and Java Implementation files",
                                     true),
              false);
        this.getCommandLine().addOption(new OptionMonolithic(this));
        this.getCommandLine().addOption(new OptionOutputDirectory(this));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        // Inits the OpenCCM's Abstract Syntax Tree,
        // the Java IDL Script Interpreter and the UML Repository.
        if(!initAST())
            return -1;
        
        ConsoleSingleton.setConsole(this.getConsole());

        try {
        	runTransformation(args[0]);
            getConsole().message("The OMG IDL 3.0, OMG CIDL and Java implementation files are generated.");
        } catch (java.lang.Exception exception) {
            getConsole().error("[ERROR] An error occurs during the reading of the UML model (" + exception.getMessage() + ").");
            return -1;
        }

        org.omg.CORBA.ComponentIR.Repository ir = org.objectweb.openccm.corba.TheInterfaceRepository.getRepository();

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.api.UML2CCM
    //
    // ==================================================================

    /**
     * Inits the UML repository.
     *
     * @return TRUE if the UML repository is initialized.
     */
    public boolean
    initUMLRepository()
    {
        try {
            org.omg.uml.UmlPackage pck = null;
            String metaModelFile = System.getProperty("uml.metamodel.file");
            pck = org.objectweb.modfact.jmi.repository.uml.UmlPackageImpl.create(metaModelFile);
            xmiImport = new org.omg.uml.xmi.UmlXMIImport();
            xmiImport.initiate(pck);
            return true;
        } catch (java.lang.Exception exception) {
            getConsole().error("[ERROR] An error occurs during the initialization of the UML repository. Check your environment.");
            exception.printStackTrace();
            return false;
        }
    }

    /**
     * Parses the UML XMI file for reading the UML Package.
     *
     * @param filename The filename of the UML XMI file.
     *
     * @throws org.xml.sax.SAXException If any parse errors occur.
     * @throws javax.xml.parsers.ParserConfigurationException If a DocumentBuilder cannot be
     *        created which satisfies the configuration requested.
     * @throws java.io.IOException If any IO errors occur.
     */
    public void
    runTransformation(String filename)
    throws
        org.xml.sax.SAXException,
        javax.xml.parsers.ParserConfigurationException,
        java.io.IOException
    {
        // Create ModelData
    	ModelData srcModel = new ModelData(new File(filename), "uml", System.getProperty("uml.metamodel.file"), "org.objectweb.modfact.jmi.repository.uml.UmlPackageImpl", "org.omg.uml.xmi.UmlXMIImport");
    	ModelData dstModel = new ModelData(null, "java", System.getProperty("java.metamodel.file"), "org.objectweb.modfact.jmi.repository.javamodel.JavaModelPackageImpl", "javamodel.xmi.JavaModelXMIExport");

        // Get URL to transformation XML files
        String jarFile = "jar:" + System.getProperty("uml.jar.file") + "!/";
    	URL [] ruleFilename = new URL[2];
    	if (this.monolithic)
    		ruleFilename[0] = new URL(jarFile + "xml/transformation/uml2javaImplMonolithic.xml");
    	else
    		ruleFilename[0] = new URL(jarFile + "xml/transformation/uml2javaImplCIF.xml");
    	ruleFilename[1] = new URL(jarFile + "xml/transformation/javaModel2JavaFiles.xml");

		// Call the UML to Java implementation transformation
		umlToJavaImplTransformation(srcModel, dstModel, ruleFilename);
		
        // Call the UML to OMG IDL 3.0 and OMG CIDL transformation
		ruleFilename = new URL[1];
        java.net.URL url = new java.net.URL(jarFile + "xml/transformation/uml2ast.xml");
		ruleFilename[0] = url;
		dstModel = new ModelData(null, "ccm-ast", null, null, null);
		umlToAstTransformation(srcModel, dstModel, ruleFilename);
    }
    
    /**
	 * Starts the transformation from the UML representation 
     * to the Java implementation files (CIF or monolithic).
     *
	 * @param srcModel The information about the source model {metamodelClass, srcFilename, modelUri}
     * @param dstModel The information about the destination model (metamodelClass, srcFilename, modelUri).
     * @param ruleFileUrls The URLs of the transformation XML files.
	 */
	public void
	umlToJavaImplTransformation(ModelData srcModel, ModelData dstModel, URL[] ruleFileUrls) {
        getConsole().message("Generating the Java implementation files...");
		try {
			// Create Engine
			MappingRuleEngineWithCollection engine = new MappingRuleEngineWithCollection();
			engine.getContext().putAttribute("outputdir", this.outputDirectory);

			// Open the input models
			PropertyValues engineArgs = new PropertyValues();
			// Create src extent (UML)
			RefPackage extent = JmiModFactModelCreateUtils.createModel(srcModel.getMetamodelURL(), srcModel.getMetamodelJmiClasses());
			JmiModFactModelCreateUtils.readModel(srcModel.getModelFile(), extent, srcModel.getModelIOClass());
			// Add model contexts to engine
			engine.getContext().addModel(srcModel.getModelUri(), new JmiModfactUML14ModelContext(extent));
            engine.getContext().setSrc(new JmiModfactUML14ModelContext(extent));
			// Add model to engine args
		    engineArgs.add(extent);

			// Open the output models
			Map dstModels = new HashMap();
            RefPackage dstExtent = null;
            ModelContext dstContext = null;
			// Create dst extent (Java)
			extent = JmiModFactModelCreateUtils.createModel(dstModel.getMetamodelURL(), dstModel.getMetamodelJmiClasses());
            dstExtent = extent;
			// Add model contexts to engine
			ModelContext modelContext = createModelContext(null, extent);
            dstContext = modelContext;
			engine.getContext().addModel(dstModel.getModelUri(), modelContext);
			// keep trace of output models
			dstModels.put(dstModel.getModelUri(), extent);

			// Create code generator initializer
            ispuml.mdaTransformation.rules.codeGenerator.CodeGeneratorXmlInitializer initializer;
            initializer = new ispuml.mdaTransformation.rules.codeGenerator.CodeGeneratorXmlInitializer();

			// Set the rule files
			for (int i = 0; i < ruleFileUrls.length; i++) {
				initializer.addRuleURL(ruleFileUrls[i]);
			}

			engine.initialize(initializer);

			// Run transformer
			// Pass the extent if there is only one src extent.
			// Pass the list of extent otherwise
			if (engineArgs.size() == 1)
				engine.runTransform(engineArgs.get(0));
			else
				engine.runTransform(engineArgs);

            extent = (RefPackage) dstModels.get(dstModel.getModelUri());
            engine.runTransform(dstExtent);
		} catch (TransformationException ex) {
			ex.printStackTrace();
		} catch (java.io.FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (java.io.IOException ex) {
			ex.printStackTrace();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * Starts the transformation from the UML representation 
     * to the OMG IDL 3.0 and OMG CIDL files.
     *
     * @param srcModel The information about the source model {metamodelClass, srcFilename, modelUri}
     * @param dstModel The information about the destination model (metamodelClass, srcFilename, modelUri).
     * @param ruleFileUrls The URLs of the transformation XML files.
	 */
	public void
	umlToAstTransformation(ModelData srcModel, ModelData dstModel, URL[] ruleFileUrls) {
        getConsole().message("Generating the OMG IDL 3.0 and OMG CIDL files...");
		try {
			// Create Engine
			MappingRuleEngineWithCollection engine = new MappingRuleEngineWithCollection();
			engine.getContext().putAttribute("outputdir", this.outputDirectory);
            
			// Open the input models
			PropertyValues engineArgs = new PropertyValues();
			// Create src extent (UML)
			RefPackage extent = JmiModFactModelCreateUtils.createModel(srcModel.getMetamodelURL(), srcModel.getMetamodelJmiClasses());
			JmiModFactModelCreateUtils.readModel(srcModel.getModelFile(), extent, srcModel.getModelIOClass());
			// Add model contexts to engine
			engine.getContext().addModel(srcModel.getModelUri(), new JmiModfactUML14ModelContext(extent));
            engine.getContext().setSrc(new JmiModfactUML14ModelContext(extent));
			// Add model to engine args
		    engineArgs.add(extent);

			// Open the output models
			Map dstModels = new HashMap();
			// Create dst extent (Java)
			Scope scope = CCMASTModelCreateUtils.createModel();
			// Add model contexts to engine
			ModelContext modelContext = new CCMASTModelContext(scope);
			engine.getContext().addModel(dstModel.getModelUri(), modelContext);
			// keep trace of output models
			dstModels.put(dstModel.getModelUri(), scope);

			// Create code generator initializer
            ispuml.mdaTransformation.rules.codeGenerator.CodeGeneratorXmlInitializer initializer;
            initializer = new ispuml.mdaTransformation.rules.codeGenerator.CodeGeneratorXmlInitializer();

			// Set the rule files
			for (int i = 0; i < ruleFileUrls.length; i++) {
				initializer.addRuleURL(ruleFileUrls[i]);
			}

			engine.initialize(initializer);

			// Run transformer
			// Pass the extent if there is only one src extent.
			// Pass the list of extent otherwise
			if (engineArgs.size() == 1)
				engine.runTransform(engineArgs.get(0));
			else
				engine.runTransform(engineArgs);
                
		} catch (TransformationException ex) {
			ex.printStackTrace();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

    /**
     * Creates the ModelContext associated to the repository.
     *
     * @param modelContextClass The name of the ModelContext class.
     * @param extent The outermost package of the repository.
     */
	protected ModelContext createModelContext(String modelContextClass, RefPackage extent) throws Exception {
		// Use the default implementation if none is specified.
		if (modelContextClass == null)
			return new JmiModfactUML14ModelContext(extent);

		try {
			Class contextClass = Class.forName(modelContextClass);
			// Try to get the constructor with an extent parameter.
			// Use the default constructor if not found.
			ModelContext context;
			try {
				java.lang.reflect.Constructor constructor = contextClass.getConstructor(new Class[] { RefPackage.class });
				context = (ModelContext) constructor.newInstance(new Object[] { extent });
			} catch (Exception ex) {
				ex.printStackTrace();
				context = (ModelContext) contextClass.newInstance();
			}

			return context;
		} catch (Exception ex) {
			throw ex;
		}
	}
	
	/**
     * Sets the monolithic flag to indicate if the Java implementation
     * will be the monolithic or CIF version.
     * 
     * @param monolithic TRUE if the Java implementation should be
     * the monolithic version.
     */
    public void
	setMonolithic(boolean monolithic) {
        this.monolithic = monolithic;
    }
    
    /**
     * Sets the output directory for the generated files.
     * 
     * @param directory The output directory for the generated files.
     */
    public void
	setOutputDirectory(String directory) {
        this.outputDirectory = directory;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        UML2CCM xmi2ir = new UML2CCM();
        xmi2ir.runMain(args);
    }

}
