/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: CCMASTModelUtil.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.ast;

import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.model.ModelUtil;
import ispuml.mdaTransformation.model.PropertyUtil;

import java.lang.reflect.InvocationTargetException;
import java.util.Comparator;
import java.util.List;

import javax.jmi.reflect.RefPackage;
import javax.jmi.reflect.RefObject;

import org.objectweb.openccm.ast.api.Scope;

/**
 * OpenCCM AST Model utilities.
 * This class implements methods dedicated to the OpenCMM AST model.
 */
public class CCMASTModelUtil implements ModelUtil {

    /** The property util used to access model properties. */
    protected PropertyUtil propUtil;

    /** The IDL concepts which are created with a 'declareXXX' method. */
    private final String [] declareConcept = {"AbstractInterface",
                                              "AbstractStorageHome",
                                              "AbstractStorageType",
                                              "CidlModule",
                                              "Component",
                                              "Event",
                                              "Interface",
                                              "LocalInterface",
                                              "Module",
                                              "PsdlModule",
                                              "StorageType",
                                              "Struct",
                                              "Union",
                                              "Value"};
    
    /** The IDL concepts which are created with a 'startXXX' method. */                                        
    private final String [] startConcept = {"Alias",
                                            "Attribute",
                                            "Composition",
                                            "Constant",
                                            "Consumes",
                                            "Emits",
                                            "Enum",
                                            "Exception",
                                            "Executor",
                                            "Factory",
                                            "FileScope",
                                            "Finder",
                                            "Home",
                                            "HomeExecutor",
                                            "Initializer",
                                            "Native",
                                            "Operation",
                                            "Provides",
                                            "PsdlOperation",
                                            "Publishes",
                                            "Segment",
                                            "StorageHome",
                                            "StorageTypeStateMember",
                                            "StorageTypeStoreDirective",
                                            "Uses",
                                            "ValueBox",
                                            "ValueMember"};


    /**
      * Default constructor.
      * Create an empty outermostRefPackage.
      */
    public CCMASTModelUtil() {
        throw new UnsupportedOperationException("Not yet implemented.");
    }

    /**
     * Default constructor.
     * Create an empty outermostRefPackage.
     */
    protected CCMASTModelUtil(Scope scope) {
        Scopes.pushScope(scope);
    }

    /**
     * Default constructor.
     * Create an empty outermostRefPackage.
     */
    public CCMASTModelUtil(PropertyUtil propUtil, Scope scope) {
        this(scope);
        this.propUtil = propUtil;
    }

    /**
     * Gets the method to invoke for the creation of the concept in the CCM AST.
     * @param scope The current scope where the concept is created.
     * @param conceptName The name of the concept to create.
     * @return The method to invoke.
     * @throws NoSuchMethodException If the concept is not an IDL concept
     *                               (No 'start' or 'declare' method assigned).
     */
    private java.lang.reflect.Method getCreationMethod(Scope scope, String conceptName) throws NoSuchMethodException {
        String methodName = conceptName;
        if (java.util.Arrays.binarySearch(declareConcept, conceptName, new StringComparator()) >= 0)
            methodName = "declare" + conceptName;
        if (java.util.Arrays.binarySearch(startConcept, conceptName, new StringComparator()) >= 0)
            methodName = "start" + conceptName;
        Class[] param = { String.class };
        java.lang.reflect.Method method = scope.getClass().getMethod(methodName, param);
        return method;
    }
    
    /**
     * Inner class for the comparison between String.
     * Comparison of the value of the Strings.
     */
    private class StringComparator implements Comparator {
        /**
         * Comparison between two String.
         * @param o1 The first object to compare.
         * @param o2 The second object to compare.
         * @return An integer which represents the difference between the both objects.
         */
        public int compare(Object o1, Object o2) {
            return ((String)o1).compareTo((String)o2);
        }
    }
    
    

    /**
     * Create an instance of specified model element.
     * @param conceptName Name of the concept to create.
     * @return The new instance.
     * @throws InstantiationException If the model element can't be created.
     */
    public Object createInstance(String conceptName) throws InstantiationException {
        try {
            java.lang.reflect.Method method = getCreationMethod(Scopes.getScope(), conceptName);
            String[] param = { "noname" };
            //System.out.println("o declare/start " + conceptName + " on " + Scopes.getScope());
            Object obj = method.invoke(Scopes.getScope(), param);
            //obj.getClass().getMethod("create", null).invoke(obj, null);
            return obj;
        } catch (NoSuchMethodException nsme) {
            throw new InstantiationException("The concept '" + conceptName + "' has no declareXXX or startXXX method.");
        } catch (IllegalAccessException e) {
            throw new InstantiationException(e.getMessage());
        } catch (IllegalArgumentException e) {
            throw new InstantiationException(e.getMessage());
        } catch (InvocationTargetException e) {
            throw new InstantiationException(e.getMessage());
        }
    }

    /** 
     * Get the associated stereotype model element.
     * @param object The object to parse.
     * @param stereotype The stereotype to search.
     * @return The stereotyped-object or null if not found.
     */
    protected RefObject getStereotype(Object object, String stereotype) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Test if the object in parameter have the stereotype.
     * @param object The object to test.
     * @param stereotype The stereotype to search.
     * @return boolean TRUE if the object is stereotyped with the stereotype.
     */
    public boolean isStereotyped(Object object, String stereotype) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Get all the stereotype of an object.
     * @param object The stereotyped-object.
     * @return List The list of the stereotypes.
     */
    public List getStereotypes(Object object) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Add a stereotype to an object.
     * @param object The object to stereotype.
     * @param stereotype The stereotype to add to the object.
     */
    public void addStereotype(Object object, String stereotype) {
        throw new UnsupportedOperationException("Not yet implemented.");
    }

    /**
     * Get the value of the tagged value associated to the stereotype.
     * @param object The object
     * @param stereotype Name of the stereotype to which the value belong
     * @param attributeName The name of the stereotype attribute (tagged value)
     * @return The value of the stereotype attribute (tagged value).
     * @throws TransformationException
     */
    public Object getStereotypeTaggedValue(Object object, String stereotype, String attributeName) throws TransformationException {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     *
     * Set the value of the tagged value associated to the stereotype.
     * @param object The object
     * @param stereotype Name of the stereotype to which the value belong
     * @param attributeName The name of the stereotype attribute (tagged value)
     * @param value The value to be set
     * @throws TransformationException
     */
    public void setStereotypeTaggedValue(Object object, String stereotype, String attributeName, Object value)
        throws TransformationException {
        throw new UnsupportedOperationException("Not yet implemented.");
    }
    
    /**
     * Is this ModelUtil owner of the specified object ?
     * If a model is owner of an object, it means that the methods of the
     * model can called on the object.
     * This method return true if this ModelUtil implementation can manage the
     * specified object (method of this class can be called on the object).
     * @param object The object to test.
     * @return true if the associated model is owner of the object, false otherwise.
     */
    public boolean isOwnerOf(Object object) {
        return true;
    }

    /**
     * Test if an object is an instance of a class.
     * @param object The object to test.
     * @param instanceName The name of the class to compare.
     * @return boolean TRUE if the classname of the object is instanceName.
     */
    public boolean isInstanceOf(Object object, String instanceName) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Is the object the model outermost package ?
     * @param object The object to test.
     * @return TRUE if the object is the outermost package.
     */
    public boolean isModelOutermostPackage(Object object) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Get the RefClass by its concept name.
     * @param conceptName The concept name.
     * @return The RefClass corresponding to the concept name.
     * @throws InvalidNameException
     */
    protected Object getRefClassByConceptName(String conceptName) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Get the refPackage by its concept name. The concept name is a "."
     * separated name. Each name denote a package name, except the last name
     * which is not use.
     * @param conceptName The concept name.
     * @return The RefPackage corresponding to the concept name.
     * @throws InvalidNameException
     */
    protected RefPackage getRefPackageByConceptName(String conceptName) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Get the refAssocaition by its association name. The concept name is a "."
     * separated name. Each name denote a package name, except the last name
     * which is not use.
     * @param conceptName The concept name.
     * @return The RefAssociation corresponding to the concept name.
     * @throws InvalidNameException
     */
    protected Object getRefAssociationByConceptName(String conceptName) {
        throw new UnsupportedOperationException("Not implemented.");
    }

    /**
     * Set the PropertyUil.
     * @param propUtil The PropertyUtil.
     */
    public void setPropertyUtil(PropertyUtil propUtil) {
        this.propUtil = propUtil;
    }

    /**
     * Get the PropertyUtil.
     * @return The PropertyUtil.
     */
    public PropertyUtil getPropertyUtil() {
        return propUtil;
    }

}
