/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: CCMPropertyUtils.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.ast;

import ispuml.mdaTransformation.utils.PropertyUtils;

import java.util.Collection;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Arrays;
import java.lang.reflect.InvocationTargetException;

/**
 * Extended version of ispuml.modelexplorer.utils.PropertyUtils.
 * This version supports List defined in IDL file (AST).
 * @author Pierre Carpentier
 */
public class CCMPropertyUtils extends PropertyUtils {

    /**
     * Set the value of the specified property of the specified bean,
     * no matter which property reference format is used, with no
     * type conversions.
     *
     * @param bean Bean whose property is to be modified
     * @param name Possibly indexed and/or nested name of the property
     *  to be modified
     * @param value Value to which this property is to be set
     *
     * @exception IllegalAccessException if the caller does not have
     *  access to the property accessor method
     * @exception IllegalArgumentException if <code>bean</code> or
     *  <code>name</code> is null
     * @exception InvocationTargetException if the property accessor method
     *  throws an exception
     * @exception NoSuchMethodException if an accessor method for this
     *  propety cannot be found
     */
    public static void setProperty(Object bean, String name, Object value)
        throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        if (isPropertyMultiple(name)) {
            setCollectionProperty(bean, name, value);
            return;
        }
        setNestedProperty(bean, name, value);
    }

    /**
     * Set the value of the specified property of the specified bean,
     * no matter which property reference format is used, with no
     * type conversions.
     *
     * @param bean Bean whose property is to be modified
     * @param name Possibly indexed and/or nested name of the property
     *  to be modified
     * @param value Value to which this property is to be set
     *
     * @exception IllegalAccessException if the caller does not have
     *  access to the property accessor method
     * @exception IllegalArgumentException if <code>bean</code> or
     *  <code>name</code> is null
     * @exception InvocationTargetException if the property accessor method
     *  throws an exception
     * @exception NoSuchMethodException if an accessor method for this
     *  propety cannot be found
     */
    public static void setCollectionProperty(Object bean, String name, Object value)
        throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        if (bean == null) {
            throw new IllegalArgumentException("No bean specified");
        }
        if (name == null) {
            throw new IllegalArgumentException("No name specified");
        }

        // Cut the name in two parts: the one before the first multiple key, and
        // the one after the first multiple key: part1[*].part2
        int multipleIndex = name.indexOf(MULTIPLE_KEY);
        if (multipleIndex < 0) {
            throw new IllegalArgumentException("No multivalue property specified (by use of [" + MULTIPLE_KEY + "] in '" + name + "'.");
        }
        int lastDelim = name.substring(0, multipleIndex).lastIndexOf(INDEXED_DELIM);
        String startName = name.substring(0, lastDelim);
        int closeDelimIndex = name.indexOf(INDEXED_DELIM2, multipleIndex);
        if (closeDelimIndex < 0) {
            throw new IllegalArgumentException("Missing closing ']' in '" + name + "'");
        }
        int nextNameIndex = name.indexOf(NESTED_DELIM, closeDelimIndex);
        if (nextNameIndex >= 0) {
            throw new UnsupportedOperationException("Multiple multivalue not supported yet.");
        }

        // Translate the provided value into a l=List if necessary
        List list;
        if (value.getClass().isArray()) {
            list = Arrays.asList((Object[]) value);
        } else if (value instanceof List) {
            list = (List) value;
        } else if (value instanceof Collection) {
            list = new ArrayList((Collection) value);
        } else {
            list = new ArrayList(1);
            list.add(value);
        }

        // Now set the requested bean property.
        // First, check the bean property type, and set it according
        // to this type
        Class type = getPropertyType(bean, startName);

        java.lang.reflect.Method method;

        // Check bean type
        if (Collection.class.isAssignableFrom(type)) {
            Collection targetCollection = (Collection) getProperty(bean, startName);
            Iterator iter = list.iterator();
            while (iter.hasNext()) {
                Object curBean = iter.next();
                targetCollection.add(curBean);
            } // end loop
        } else if ((method = getAddMethod(list, type)) != null) {
            // Checks if the type have a 'add' method with a parameter type
            // got from the source list
            Iterator iter = list.iterator();
            while (iter.hasNext()) {
                Object target = getProperty(bean, startName);
                Object curBean = iter.next();
                Object param[] = { curBean };
                method.invoke(target, param);
            }
        } else { // error !
            throw new IllegalArgumentException("Name '" + startName + "' doesn't denote a multivalue property.");
        }

    }

    /**
     * Get the 'add' method of a list. This methods is compliant with the CORBA list (sequence).
     * @param list The list which contains the 'add' method.
     * @param type The type of the object to add to the list.
     * @return The method to call to add the new object to the list.
     */
    private static java.lang.reflect.Method getAddMethod(List list, Class type) {
        if (list.size() > 0) {
            java.lang.reflect.Method methods [] = type.getMethods();
            for (int i=0 ; i<methods.length ; i++) {
                if (methods[i].getName().equals("add")) {
                    Class paramType [] = methods[i].getParameterTypes();
                    if (paramType.length == 1 && paramType[0].isAssignableFrom(list.get(0).getClass()))
                        return methods[i];
                }
            }
        } else {
            try {
                return type.getMethod("add", new Class[0]);
            } catch (Exception exception) {
                return null;
            }
        }
        return null;
    }

}