/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

--------------------------------------------------------------------
$Id: PreprocessorCPP.java,v 1.1 2004/02/05 20:29:57 rouvoy Exp $
====================================================================*/

package org.objectweb.util.cpp.lib;

import java.io.File;
import java.io.FileReader;
import java.io.OutputStream;
import java.io.PrintStream;

import org.objectweb.util.misc.api.ExceptionWrapper;


/**
 * This provides a wrapper to external cpp preprocessors.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.2
 */
public class PreprocessorCPP
     extends PreprocessorBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The path to the external C preprocessor used. */
    private String cppPath_;
 
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public PreprocessorCPP() {
        // The default cppPath is cpp.
        this("cpp");
    }

    /**
     * The constructor with the path to the external C preprocessor used.
     *
     * @param cppPath The path to the external C preprocessor used.
     */
    public PreprocessorCPP(String cppPath) {
        cppPath_ = cppPath;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorOperations
    //
    // ==================================================================

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param tmpFile The file containing the output file.
     *
     * @return True if OK.
     */
    public boolean preprocess(String fileName, File tmpFile) {
        // Prepares the command line.

        // Allocates the string command array passed as argument
        // to the exec() method.

        int nb_arguments = arguments_.size();
        String[] cmd = null;

        // This array length is equals to the number of arguments
        // plus 3 (i.e. cpp path, input and output file names)
        // plus 1 if the -P option is required.

        if(!getGenerateLineInformation()) {
            cmd = new String[3 + nb_arguments];
        } else {
            cmd = new String[4 + nb_arguments];
        }

        // Adds the cpp path to the command array.
        int i = 0;
        cmd[i] = cppPath_; i++;

        // Adds the -P option if required.
        if(getGenerateLineInformation()) {
            cmd[i] = "-P"; i++;
        }

        // Adds all arguments.
        for(int j=0; j<nb_arguments; j++) {
            cmd[i] = (String)arguments_.elementAt(j); i++;
        }

        // Adds the input file name.
        cmd[i] = fileName; i++;

        // Adds the output file name.
        cmd[i] = tmpFile.toString(); i++;

        // Executes the command.
        Process cpp = null;
        try {
            cpp = Runtime.getRuntime().exec(cmd);
        } catch(java.io.IOException exc) {
            // Wraps any java.io.IOException exception.
            throw new ExceptionWrapper(exc);
        }

        int status = 0;
        try {
            // Waits for the process completion.
            status = cpp.waitFor();
        } catch(java.lang.InterruptedException exc) {
            // Wraps any java.lang.InterruptedException exception.
            throw new ExceptionWrapper(exc);
        }

        // Dumps its standard input and error streams.
        IOHelper.dump(cpp.getInputStream(), getConsole().getOutputStream());
        IOHelper.dump(cpp.getErrorStream(), getConsole().getErrorStream());

        // Returns true if the status == 0.
        return status == 0;
    }

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param output The output stream.
     */
    public boolean preprocess(String fileName, OutputStream output) {
        // Preprocesses the file.
        File tmpFile = preprocess(fileName);
        if(tmpFile == null)
            return false;

        try {
            // Dumps the content of the temp file.
            IOHelper.dump(new FileReader(tmpFile), new PrintStream(output));
        } catch(java.io.IOException exc) {
            // Wraps any java.io.IOException exception.
            throw new ExceptionWrapper(exc);
        } finally {
            // Always destroys the temp file.
            tmpFile.delete();
        }

        return true;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.Preprocessor
    //
    // ==================================================================
}
